#!/usr/bin/env python3
#generate pass function for PyBLD
# Copyright (C) 2000-2017 Hiroshi Watabe watabe@cyric.tohoku.ac.jp
import os
import sys
import re
from numpy.distutils.misc_util import get_numpy_include_dirs
numpy_incdir = get_numpy_include_dirs()

narg = len(sys.argv)
if narg!=3:
	print("Usage:gen_pass_pybld.py source-file main-routine-name")
	sys.exit()
user_file = sys.argv[1]
fp = open(user_file,'r')
lines = fp.readlines()
fp.close();
prog_name = sys.argv[2]
pass_file_name = "pybld_pass_%s.c" % (prog_name)
swig_file = "%s.i" % (prog_name)
makefile_file = "makefile.%s" % (prog_name)
#print len(lines),sys.argv[2]
sstring = "%s.*\(.*\)" % (sys.argv[2])
sample = re.compile(sstring)
typemat = []
for i in range(len(lines)):
	try:
		loc = sample.search(lines[i]).start()
	except:
		pass
	else:
		#find return type
		if lines[i][0:loc].find('double')==-1:
			return_type = 'float'
		else:
			return_type = 'double'
		line1 = lines[i][loc:-1];
		sample = re.compile('\(')
		loc = sample.search(line1).start()
		line2 = line1[(loc + 1):];
		sample = re.compile('\)')
		loc = sample.search(line2).start()
		line3 = line2[0:loc]
		varmat =  line3.split(',')
		varmat = varmat[1:(len(varmat)-2)] #first argument is ndim and last two arguments are num_col and num_row which user must supply
		n_var = len(varmat) #number of variables which user define
		for j in range(n_var):
			loc =  varmat[j].find('*')
			if varmat[j].find('double')==-1:
				typemat.append('float')
			else:
				typemat.append('double')
				
			varmat[j] = varmat[j][(loc + 1):len(varmat[j])];

		break

#generate C source code for pybld_pass
fp = open(pass_file_name,'w');
fp.write("/* Generated by gen_pass_pybld.py. */\n");
fp.write("#include <Python.h>\n#include <numpy/arrayobject.h>\n\n");
fp.write("extern int float2array2d(float*,PyArrayObject*);\nextern int float2array1d(float*,PyArrayObject*);\nextern float *array2float(PyArrayObject*);\n");
fp.write("extern int double2array2d(double*,PyArrayObject*);\nextern int double2array1d(double*,PyArrayObject*);\nextern double *array2double(PyArrayObject*);\n");
if return_type=='float':
	sline = "float *%s(int*," % (prog_name)
else:
	sline = "double *%s(int*," % (prog_name)
for i in range(n_var):
	if typemat[i]=='float':
		str_fl = "float*,"
	else:
		str_fl = "double*,"
	sline = sline + str_fl;
sline = sline + "int*,int*);\n\n"
fp.write(sline);
sline = "PyArrayObject *pass_src(PyArrayObject "
for i in range(n_var - 1):
	aline = "*%s_ar, PyArrayObject " % (varmat[i])
	sline = sline + aline
aline = "*%s_ar)" % (varmat[n_var - 1])
sline = sline + aline
fp.write(sline);
if return_type=='float':
	fp.write("{\nint *ndim,num_row,num_col;\nfloat *fmat;\nnpy_intp dimensions[2];\nPyArrayObject *pass_result;\n");
else:
	fp.write("{\nint *ndim,num_row,num_col;\ndouble *fmat;\nnpy_intp dimensions[2];\nPyArrayObject *pass_result;\n");

sline = "";
for i in range(n_var):
	if typemat[i]=='float':
		aline = "float *%s;\n" % (varmat[i]);
	else:
		aline = "double *%s;\n" % (varmat[i]);
	sline = sline + aline
fp.write(sline);
fp.write("\n\nimport_array();\nndim = (int *)malloc(sizeof(int)*%d*2);\n" % (n_var))
for i in range(n_var):
	fp.write("ndim[%d] = %s_ar->dimensions[0];\n" % (2*i,varmat[i]))
	fp.write("ndim[%d] = 1;\n" % (2*i + 1)) #assume given variables are 1-dimentional vector
for i in range(n_var):
	if typemat[i]=='float':
		fp.write("%s = array2float(%s_ar);\n" % (varmat[i],varmat[i]))
	else:
		fp.write("%s = array2double(%s_ar);\n" % (varmat[i],varmat[i]))
sline = "fmat = %s(ndim," % (prog_name)
for i in range(n_var):
	aline = "%s," % (varmat[i])
	sline = sline + aline
sline = sline + "&num_row,&num_col);\n"
fp.write(sline);
for i in range(n_var):
	fp.write(" free(%s);\n" % (varmat[i]))

if return_type=='float':
	fp.write("dimensions[0] = num_row;\ndimensions[1] = num_col;\npass_result = (PyArrayObject *)PyArray_SimpleNew(2,(npy_intp*)dimensions,PyArray_FLOAT);\n");
	fp.write("if(dimensions[1]>1){float2array2d(fmat,pass_result);}\nif(dimensions[1]==1){float2array1d(fmat,pass_result);}\nfree(fmat);\nfree(ndim);\nreturn (PyArrayObject *)PyArray_Return(pass_result);\n}\n");
else:
	fp.write("dimensions[0] = num_row;\ndimensions[1] = num_col;\npass_result = (PyArrayObject *)PyArray_SimpleNew(2,(npy_intp*)dimensions,PyArray_DOUBLE);\n");
	fp.write("if(dimensions[1]>1){double2array2d(fmat,pass_result);}\nif(dimensions[1]==1){double2array1d(fmat,pass_result);}\nfree(fmat);\nfree(ndim);\nreturn (PyArrayObject *)PyArray_Return(pass_result);\n}\n");
fp.close()
######################
#print out swig i file
#####################
fp = open(swig_file,'w');
sline = "PyArrayObject *pass_src(PyArrayObject ";
for i in range(n_var - 1):
	aline = "*%s_ar,PyArrayObject " % (varmat[i]);
	sline = sline + aline
aline = "*%s_ar);\n" % (varmat[n_var - 1]);
sline = sline + aline
fp.write("/* Generated by gen_pass_pybld.py. */\n");
percent = "%"
fp.write("%smodule %s\n\n" % (percent,prog_name))
fp.write("%s{\n#include <numpy/arrayobject.h>\n" % (percent))
fp.write(sline);
fp.write("%s}\n\n" % (percent))
fp.write("%sinit %s{\n\timport_array();\n%s}\n\n" % (percent,percent,percent))
fp.write("%stypemap(python,in) PyArrayObject * {\n  $1 = (PyArrayObject *)$input;\n}\n" % (percent))
fp.write("%stypemap(python,out) PyArrayObject * {\n  $result = (PyObject *)$1;\n}\n\n" % (percent))
fp.write("extern "+sline);
fp.close()
######################
#print out makefile
#####################
fp = open(makefile_file,'w');
fp.write("#Generated by gen_pass_pybld.py\n");
fp.write("SWIG       = swig\nSO      = .so\n");
fp.write("MODULE     = %s\nINTERFACE  = %s.i\n" % (prog_name,prog_name));
fp.write("SRCS       = pybld_pass_%s.c %s bld_util.c\n" % (prog_name,user_file));
fp.write("ISRCS      = $(INTERFACE:.i=_wrap.c)\nTARGET     = _$(MODULE)$(SO)\n");
fp.write("OBJS      = $(SRCS:.c=.o)\nIOBJS     = $(ISRCS:.c=.o)\nLDSHARED = $(CC) -shared\n");
fp.write("CCSHARED=       -fpic\nPYTHON_INCLUDE= -DHAVE_CONFIG_H -I/usr/include/python3.6m -I/usr/include/python3.6m\n");
fp.write("CFLAGS = -c -fpic -DHAVE_CONFIG_H -I/usr/include/python3.6m -I/usr/include/python3.6m -I%s -g\n" % (numpy_incdir[0]));
fp.write("LIBS = \n\n$(TARGET): $(OBJS) $(IOBJS)\n");
fp.write("\t$(LDSHARED) $(OBJS) $(IOBJS) $(LIBS) -o $(TARGET)\n");
fp.write("$(ISRCS): $(INTERFACE)\n\t$(SWIG) -python $(SWIGOPT) $(INTERFACE)\n");
fp.write("$(IOBJS): $(ISRCS)\n\t$(CC) -c $(CCSHARED) $(CFLAGS) $(ISRCS) $(INCLUDE) $(PYTHON_INCLUDE)\n");
fp.write("clean::\n\trm -f $(OBJS) $(IOBJS) $(TARGET)\n");
fp.close()
#finish
print("Please copy bld_util.c and bld.h from PyBLD source tree")
