/* convg 1 and 2 subroutines for BLD fitting */
/* Copyright (C) 2000 - 2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
/* $Date: 2004-03-08 09:45:33 +0900 (月, 08  3月 2004) $ */
#include <math.h>
#include "bld.h"

void convg1(double *gvec,double *qvec, double *infmat)
{
  double det;
  /*  INVERT INFORMATION MATRIX*/
  dminv_(infmat,fit_puse,&det,gvec,qvec);
  if(conv==1){
    printf("** Convergence has occurred **\n");
  }
  if(conv!=1){
    printf("....Aborted before convergence !\n");
  }
  if(fit_weight){
    printf("Weighted regression\n");
  }
  printf("Parameter   Estimate       Standard\n");
  printf("                            Error\n");
}

void convg2(double *parm,double *qvec,double *infmat)
{
/*
C   CONVG2(B,Q,S,iptr) - THIS SUBROUTINE PROCESSES THE SECOND HALF OF THE 
C	OPERATIONS AFTER NON-LINEAR REGRESSION HAS REACHED CONVERGENCE.
C	IT CALCULATES THE STANDARD ERRORS FROM THE INVERSE OF THE INFORMATION
C	MATRIX AND THE RESIDUAL SUM OF SQUARES AND CALLS STAT_report TO PRODUCE
C	THE STATISTICAL SUMMARY.   
C
C  ARGUMENTS:
C	B  - VECTOR OF PARAMETER ESTIMATES
C	Q - WORK VECTORS FOR EXTRACTION OF DIAGONAL
C	S - INVERSE OF INFORMATION MATRIX   
C                                 1/2
C   ASYMPTOTIC STD ERROR OF B  = S   * STD ERROR ABOUT THE CURVE
C                            j    jj
*/
  int df[2];
  double s4,se;
  int ioffs,i,j;
  float *ffit_obs,*ffit_wt;
  /*
    Q CONTAINS VARIANCES
  */
  diag_(qvec,infmat,fit_puse);
  s4 = sqrt(cur_ss/(double)(fit_ndat - fit_puse));/*	!STANDARD ERROR ABOUT THE CURVE*/
  /*
C  RC31JUL86   - CALCULATE CORRELATION MATRIX AND STORE
C			IN A VARIABLE CALLED CORRMAT
  */
  ioffs=0;
  for(i=0;i<fit_puse;i++){
    for(j=0;j<fit_puse;j++){
      fit_cormat[ioffs] = infmat[ioffs]/sqrt(qvec[i]*qvec[j]); /*	!CALCULATE CORRELATION*/
	ioffs++;
    }
  }
  /*
 PRINT OUT COEFFICIENTS AND THEIR STANDARD ERRORS
  */
  j = 0;
  for(i=0;i<fit_npar;i++){
    printf("  %d       %lg   ",i+1,parm[i]);
    if (fit_fixmat[j]==(i + 1)){
      se = sqrt(qvec[j])*s4;
      printf("%lg\n",se);
      j++;
    }
    else{
      se = 0.0;
      printf("   fixed\n");
    }
    fit_semat[i] = se;
  }
  /*
  PRINT OUT THE STATISTICAL SUMMARY
  */
  df[0] = fit_puse;
  df[1] = fit_ndat - 1 - fit_puse;
  /* double to float*/
  ffit_obs = (float *)malloc(sizeof(float)*fit_ndat);
  for(i=0;i<fit_ndat;i++){
    ffit_obs[i] = (float)fit_obs[i];
  }
  if(fit_weight){
    ffit_wt = (float *)malloc(sizeof(float)*fit_ndat);
    for(i=0;i<fit_ndat;i++){
      ffit_wt[i] = (float)fit_wt[i];
    }
  }
  else{
    ffit_wt = NULL;
  }
  stat_report(ffit_obs,1,fit_ndat,ffit_wt,fit_weight,1,cur_ss,s4,df);
  free(ffit_obs);
  free(ffit_wt);
  /*
  copy parameter values back
  */
  for(i=0;i<fit_npar;i++){
    fit_pcur[i] = parm[i];
  }
}
