/* fitting for BLD */
/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
#include "bld.h"

int fit_weight = 0;
int fit_const = 0;
double fit_dp = 0.001;
PyObject *fitfunc = NULL;
int fit_npar,fit_ndat,fit_puse;
double *fit_calc;
double *fit_calc_pre;
double *fit_diff;
double *fit_tm,*fit_pmat;
PyArrayObject *fit_tm_ar; /* for calling function */
PyArrayObject *fit_par_ar; /* for calling function */
double *fit_obs;
double *fit_wt;
double *fit_semat;
unsigned char *fit_fixmat;
int num_iter;
unsigned char fit_func = 2; /* type of fitting default is general fitting */
unsigned char show_flag = 1; /* show detail results*/
int fit_iterations=100;
int fit_subiterations = 100;
float fit_converge = 0.1;
int ktlx;
unsigned char conv;
double cur_ss;
double *fit_cormat;
double *fit_pcur; /*current param */

extern PyArrayObject *callfitfunc(PyArrayObject*,PyArrayObject*, void*);
void start_fit(double*);

PyArrayObject *fit(PyObject *PyFunc, PyArrayObject *tm_ar, PyArrayObject *obs_ar, PyArrayObject *par_ar,  PyArrayObject *wt_ar, PyArrayObject *fix_ar)
{
  double *fixmat;
  int i;
  npy_intp dimensions[1];
  double *gvec,*qvec,*vvec,*infmat,*ffmat,*tmpp;
  PyArrayObject *result;

  import_array(); /* to make share array library */
  fit_puse = 0;//initialize
  fitfunc = PyFunc;
  fit_tm = array2double(tm_ar);
  fit_pmat = array2double(par_ar);
  fit_obs = array2double(obs_ar);
  if(fit_weight)fit_wt = array2double(wt_ar);
  fit_npar = par_ar->dimensions[0];
  fit_ndat = tm_ar->dimensions[0];
  if(fit_func==1){/* derivatives are defined */
    fit_calc = (double *)malloc(sizeof(double)*fit_ndat*(1 + fit_npar));
  }
  else fit_calc = (double *)malloc(sizeof(double)*fit_ndat);
  fit_calc_pre = (double *)malloc(sizeof(double)*fit_ndat);
  fit_diff = (double *)malloc(sizeof(double)*fit_ndat*fit_npar);
  memset(fit_diff,0,sizeof(double)*fit_ndat*fit_npar);
  dimensions[0] = fit_ndat;
  fit_tm_ar = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_DOUBLE);
  tmpp = (double *)fit_tm_ar->data;
  for(i=0;i<fit_ndat;i++){
    tmpp[i] = fit_tm[i];
  }
  /* replace fit_tm */
  free(fit_tm);
  fit_tm = (double *)fit_tm_ar->data;

  dimensions[0] = fit_npar;

  fit_par_ar = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_DOUBLE);
  tmpp = (double *)fit_par_ar->data;
  for(i=0;i<fit_npar;i++){
    tmpp[i] = fit_pmat[i];
  }
  /* replace fit_pmat */
  free(fit_pmat);
  fit_pmat = (double *)fit_par_ar->data;
  /* fixed matrix */
  fit_fixmat = (unsigned char *)malloc(fit_npar);
  if(fit_const==0){ //no constrain
    for (i=0,fit_puse;i<fit_npar;i++){
      fit_fixmat[i] = i + 1;
      fit_puse ++;
    }
  }
  else{
    fixmat = array2double(fix_ar);
    for(i=0;i<fit_npar;i++){
      if(fixmat[i]>=1.0){
	fit_fixmat[i] = 0;
      }
      else{
	fit_fixmat[i] = 1;
      }
    }
    for(i=0,fit_puse=0;i<fit_npar;i++){
      if(fit_fixmat[i]){
	fit_puse++;
	fit_fixmat[fit_puse - 1] = i + 1;
      }
    }
    if(fit_puse<fit_npar) fit_fixmat[fit_puse] = 0;
  }
  fit_semat = (double *)malloc(sizeof(double)*fit_npar);
  gvec = (double *)malloc(sizeof(double)*fit_npar);
  qvec = (double *)malloc(sizeof(double)*fit_npar);
  vvec = (double *)malloc(sizeof(double)*fit_npar);
  infmat = (double *)malloc(sizeof(double)*fit_npar*fit_npar);
  ffmat = (double *)malloc(sizeof(double)*fit_npar*fit_npar);
  fit_cormat = (double *)malloc(sizeof(double)*fit_puse*fit_puse);
  fit_pcur = (double *)malloc(sizeof(double)*fit_npar);
  if((gvec==NULL)||(qvec==NULL)||(vvec==NULL)||(infmat==NULL)||(ffmat==NULL)||(fit_cormat==NULL)||(fit_pcur==NULL)){
    free(gvec);
    free(qvec);
    free(vvec);
    free(infmat);
    free(ffmat);
    free(fit_diff);
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory");
    return NULL;
  }
  memcpy(fit_pcur,fit_pmat,sizeof(double)*fit_npar);
  memset(gvec,0,sizeof(double)*fit_npar);
  memset(qvec,0,sizeof(double)*fit_npar);
  memset(vvec,0,sizeof(double)*fit_npar);
  memset(infmat,0,sizeof(double)*fit_npar*fit_npar);
  memset(ffmat,0,sizeof(double)*fit_npar*fit_npar);
  memset(fit_cormat,0,sizeof(double)*fit_puse*fit_puse);
  /* start fitting */
  start_fit(fit_pmat);
  while(1){
    if(getdir(fit_pmat,fit_func)){
      free(gvec);
      free(qvec);
      free(vvec);
      free(infmat);
      free(ffmat);
      free(fit_diff);
      PyErr_SetString(PyExc_ValueError, "Failed to fit");
      return NULL;
    }
    if(iter1(gvec,qvec,vvec,infmat,ffmat)){
      free(gvec);
      free(qvec);
      free(vvec);
      free(infmat);
      free(ffmat);
      free(fit_diff);
      PyErr_SetString(PyExc_ValueError, "Failed to fit");
      return NULL;
    }
    if(ktlx==10) break;
    if(iter2(fit_pmat,gvec,qvec,vvec,infmat,ffmat)){
      free(gvec);
      free(qvec);
      free(vvec);
      free(infmat);
      free(ffmat);
      free(fit_diff);
      PyErr_SetString(PyExc_ValueError, "Non-singular information matrix encountered (determinant<1E-20)");
      return NULL;
    }
    if(subit(fit_pmat,gvec,qvec)){
      free(gvec);
      free(qvec);
      free(vvec);
      free(infmat);
      free(ffmat);
      free(fit_diff);
      PyErr_SetString(PyExc_ValueError, "Failed to fit");
      return NULL;
    }
    if(testcg(fit_pmat,qvec)){
      free(gvec);
      free(qvec);
      free(vvec);
      free(infmat);
      free(ffmat);
      free(fit_diff);
      PyErr_SetString(PyExc_ValueError, "Failed to fit");
      return NULL;
    }
  }
  convg1(gvec,qvec,infmat);
  convg2(fit_pmat,qvec,infmat);
  if(conv!=1){
    free(gvec);
    free(qvec);
    free(vvec);
    free(infmat);
    free(ffmat);
    free(fit_diff);
    PyErr_SetString(PyExc_ValueError, "maximum iterations reached");
    return NULL;
  }
  /* finish fitting */
  /* prepare result matrix */
  dimensions[0] = fit_npar + fit_npar + fit_puse*fit_puse + 7;
    /*param, se cormat report mat */
  result = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_DOUBLE);
  for(i=0;i<fit_npar;i++){
    (*(double *)(result->data +i*result->strides[0])) = (double)fit_pmat[i];
  }
  for(i=0;i<fit_npar;i++){
    (*(double *)(result->data +(i + fit_npar)*result->strides[0])) = (double)fit_semat[i];
  }
  for(i=0;i<fit_puse*fit_puse;i++){
    (*(double *)(result->data +(i + 2*fit_npar)*result->strides[0])) = fit_cormat[i];
  }
  for(i=0;i<7;i++){
    (*(double *)(result->data +(i + 2*fit_npar + fit_puse*fit_puse)*result->strides[0])) = report_mat[i];
  }

  free(gvec);
  free(qvec);
  free(vvec);
  free(infmat);
  free(ffmat);
  free(fit_diff);
  free(fit_pcur);
  free(fit_cormat);
  free(fit_semat);
  free(fit_tm);
  free(fit_obs);
  free(fit_calc);
  free(fit_calc_pre);
  free(fit_fixmat);
  return (PyArrayObject *)PyArray_Return(result);
}

void start_fit(double *par)
{
  int i,j;

  printf("Number of parameters = %d\n",fit_npar);
  printf("Initial parameter estimates :\n");
  j = 1;
  for(i=0;i<fit_npar;i++){
    printf("Parameter %d = %lg",i+1,par[i]);
    if(fit_fixmat[j - 1]==(i + 1)){ j++;printf("\n");}
    else printf(" fixed\n");
  }
  if(fit_weight){
    printf("Weighted regression\n");
  }
  if(eval_func(par,fit_func)==0){
    sumsq(&cur_ss);
    printf("Initial sum of squares = %lg\n",cur_ss);
    ktlx = 3;
    num_iter = 0;
    conv = 0;
  }
}


/* statistics report */
void stat_report(float *indep_y,int n0,int n1,float *wt,int weight,int cst,double ssar, double sees, int *df){
  float ssa,see;
  float stot,sumwt,ssum,w,yy,ff,r2,r;
  int i;
  /*
    COMPUTE TOTAL WEIGHTED SUM OF SQUARED 
  */
  stot = 0;
  sumwt = 0.0;
  ssum = 0;
  w = 1.0;
  for(i=n0;i<=n1;i++){
    yy = indep_y[i - 1];
    if(weight) w = wt[i - 1];
    sumwt += w;
    ssum += yy*w;
    stot += w*yy*yy;
  }
  if(cst) stot = stot - ssum*ssum/sumwt;

  see = (float)sees;
  ssa = (float)ssar;

  if(show_flag)printf("Std error of the estimate : %g\n",see);
  if(show_flag)printf("Sum of squares : %g\n",ssa);
  /*  COMPUTE F VALUE AND PRINT */
  if ((ssar==0.0)||(df[0]==0)||(df[1]==0)) ff = 0.0;
  else{
    ff = (float)(((stot - ssar)/(float)df[0])/(ssar/(float)df[1]));
  }
  if(show_flag)printf("F value (%d,%d) : %g\n",df[0],df[1],ff);
  /*   EVALUATE R-SQUARED AND CORRELATION COEFFICIENT*/
  if (stot== 0.0) r2 = 0.0;
  else{
    r2 = (float)((stot - ssar)/stot);
  }
  report_mat[0] = see;
  report_mat[1] = ssa;
  report_mat[2] = (double)df[0];
  report_mat[3] = (double)df[1];
  report_mat[4] = (double)ff;
  report_mat[5] = (double)r2;
  if(show_flag)printf("R square : %g\n",r2);
  if(r2<0.0){
		  /*
C   R-SQUARED IS NEGATIVE.  THIS MEANS THAT THERE IS MORE DEVIATION REMAINING
C	ABOUT THE REGRESSION CURVE THAT THAN THERE IS INHERENT VARIATION IN
C	THE DATA (OR VARIATION ABOUT ITS WEIGHTED MEAN)
		  */
    if(show_flag)printf("Fit not satisfactory.\n");

  }
  else{
    r = (float)sqrt((double)r2);
    if(show_flag)printf("Correlation coefficient : %g\n",r);
    report_mat[6] = (double)r;
  }
}
