/* fit subroutines for BLD fitting */
/* Copyright (C) 2000 - 2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
#include "bld.h"

extern PyArrayObject *callfitfunc(PyArrayObject*,PyArrayObject*, void*);

int eval_func(double *par, int func)
{
  PyArrayObject *res_ar;
  int i,j;
  double yy,t,ex;
  int ires;

  //  memcpy(fit_pmat,par,sizeof(double)*fit_npar);
  /*exponential */
  if(func==0){
    for(i=0;i<fit_ndat;i++){
      t = fit_tm[i];
      yy = 0.0;
      if(fit_npar%2==1) yy = par[fit_npar - 1];
      for(j=1;j<fit_npar;j+=2){
	ex = par[j]*t;
	if(ex>38.0) ex = 38.0;
	yy += par[j-1]*exp(ex);
      }
      fit_calc[i] = yy;
    }
    return(0);
  }
  
  res_ar = callfitfunc(fit_tm_ar,fit_par_ar,(void *)fitfunc);
  if(res_ar==NULL){
    ires = 1; //error occurs
    return(ires);
  }
  if(func==1){/* if derivatives are defined */
    for(i=0;i<fit_ndat*(fit_npar + 1);i++){
      if(res_ar->descr->type_num == PyArray_FLOAT){
	fit_calc[i] = (float)(*(float *)(res_ar->data + i*res_ar->strides[0]));
      }
      else{
	if(res_ar->descr->type_num == PyArray_DOUBLE){
	  fit_calc[i] = (double)(*(double *)(res_ar->data + i*res_ar->strides[0]));
	}
      }
    }
  }
  else{
    for(i=0;i<fit_ndat;i++){
      if(res_ar->descr->type_num == PyArray_FLOAT){
	fit_calc[i] = (float)(*(float *)(res_ar->data + i*res_ar->strides[0]));
      }
      else{
	if(res_ar->descr->type_num == PyArray_DOUBLE){
	  fit_calc[i] = (double)(*(double *)(res_ar->data + i*res_ar->strides[0]));
	}
      }
    }
  }
  ires = 0; // no error occurs
  return(ires);
}

void sumsq(double *ss)
{
  double dx,dx2;
  int i;

  *ss = 0.0;
  for(i=0;i<fit_ndat;i++){
    dx = fit_calc[i] - fit_obs[i];
    if(fabs(dx)>1.0e18) dx = 1e18;
    dx2 = dx*dx;
    if(fit_weight) dx2 *= fit_wt[i];
    *ss += dx2;
    if(*ss>1e35) return;
  }
}

int getdir(double *par,int func)
{
  int i,j,ii;
  double t,cur_p,dpar;
  double ex;
  if(func==0){
    /*  EXPONENTIALS */
    for(i=0;i<fit_ndat;i++){
      t = fit_tm[i];
      for(j=1;j<fit_npar;j+=2){
	ex = par[j]*t;
	if(ex>38.0) ex = 38.0;
	ex = exp(ex);
	fit_diff[(j - 1)*fit_ndat + i] = ex;
	fit_diff[j*fit_ndat + i] = par[j - 1]*t*ex;

      }
      if(fit_npar%2==1) fit_diff[(fit_npar - 1)*fit_ndat + i] = 1.0;
    }
    return(0);
  }
  for(i=0;i<fit_puse;i++){
    ii = (int)fit_fixmat[i] - 1;
    cur_p = par[ii];
    dpar = cur_p*fit_dp;
    if(dpar==0) dpar = fit_dp;
    par[ii] = cur_p + dpar;
    if(eval_func(par,2)){//error occur
      return(1);
    }
    if(func==1){ /* define derivatives */
      for(j=0;j<fit_ndat;j++){
	fit_diff[ii*fit_ndat + j] = fit_calc[fit_ndat*(i+1) + j];
      }
    }
    else{
      memcpy(fit_calc_pre,fit_calc,sizeof(double)*fit_ndat); /* reserve previous fitting results */
      par[ii] = cur_p;
      if(eval_func(par,2)){//error occurs
	return(1);
      }
      for(j=0;j<fit_ndat;j++){
	fit_diff[ii*fit_ndat + j] = (fit_calc_pre[j] - fit_calc[j])/dpar;
      }
    }
  }
  return(0);
}
