/* Interpolation BLD version */
/* Copyright (C) 2000-2020 Hiroshi Watabe watabe@cyric.tohoku.ac.jp */
/* $Id: bld_interpolate.c,v 3acbaa38e8f4 2020/01/13 01:53:32 watabe $ */
#include "bld.h"

PyArrayObject *interpolate(PyArrayObject *tm_array, PyArrayObject *cnt_array, PyArrayObject *out_array)
{
  int in_n;
  int out_n;
  float *tm,*cnt,*newtm,*newcnt;
  PyArrayObject *result;
  npy_intp dimensions[1];

  import_array(); /* to make share array library */
  in_n = tm_array->dimensions[0];
  if(in_n<2){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare array. number of input array must have more than 2 elements");
    return NULL;
  }
  out_n = out_array->dimensions[0];

  dimensions[0] = out_n;
  result = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_FLOAT);
  newcnt = (float *)result->data;

  if((tm = array2float(tm_array))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable tm");
    return NULL;
  }
  if((cnt = array2float(cnt_array))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable cnt");
    return NULL;
  }
  if((newtm = array2float(out_array))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable newtm");
    return NULL;
  }

  interpolate_(&in_n,tm,cnt,&out_n,newtm,newcnt);
  free(cnt);
  free(newtm);
  free(tm);
  return (PyArrayObject *)PyArray_Return(result);
}
