/* fit subroutines for BLD fitting */
/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
/* $Date: 2004-03-02 08:49:18 +0900 (火, 02  3月 2004) $ */
#include "bld.h"

int iter1(double *sens_vec,double *qvec, double *vstep, double *infmat, double *fwork){
  double cur_w,r1;
  int i,j;

  memset(infmat,0,fit_puse*fit_puse*sizeof(double));
  memset(vstep,0,sizeof(double)*fit_puse);
  num_iter++;

  /*C  COMPUTE MATRIX OF SUMS OF CROSS PRODUCTS OF DERIVATIVES*/
  /*	L8=LOC(8)	!WEIGHTS
	L2=LOC(2)	!MEASURED DATA
	L3=LOC(3)	!FITTED DATA
	L5=LOC(5)	!DERIVATIVES
  */
  cur_w = 1.0;
  for(i=0;i<fit_ndat;i++){
    if(fit_weight) cur_w = fit_wt[i];
    r1 = cur_w*(fit_obs[i] - fit_calc[i]);
    for(j=0;j<fit_puse;j++){
      sens_vec[j] = fit_diff[(fit_fixmat[j] - 1)*fit_ndat + i];
    }
    dmmulk_(qvec,sens_vec,r1,1,fit_puse);
    dmadd_(vstep,vstep,qvec,1,fit_puse);

    dmmulk_(qvec,sens_vec,cur_w,1,fit_puse);
    dmmul_(fwork,sens_vec,qvec,fit_puse,1,fit_puse);
    dmadd_(infmat,infmat,fwork,fit_puse,fit_puse);
  }
  if(num_iter>fit_iterations) ktlx = 10;
  if(conv!=0) ktlx = 10;
  return 0;
}

int iter2(double *para,double *gvec,double *qvec,double *gstep,double *infmat,double *invinf){
  /*
 - THIS LOGICAL*1 FUNCTION PERFORMS THE SECOND
	HALF OF THE MAIN GAUSS-NEWTON STEP OF NON-LINEAR REGRESSION.
	IT IS THE CONTINUATION OF THE PROCESSING IN THE MODULE ITER1.
	THE INFORMATION MATRIX S IS INVERTED IN F AND THE PARAMETER
	UPDATE IS CCOMPUTED.   MUCH OF THE REST OF THE CODE COMES
	FROM A PREVIOUS IMPLEMENTATION WITH FIparaONACCHI MARQUARDTING
	AND HAS BEEN REMOVED IN FAVOR OF THE SIMPLER STEP-HALVING.
	THIS FUNCTION RETURNS TRUE IF A ZERO DETERMINANT IS DETECTED.

   ARGUMENTS:
	NP - INTEGER NUMBER OF PARAMETERS
	para - REAL*8 VECTOR OF PARAMETER ESTIMATES (NPx1)
	gvec - REAL*8 WORK VECTOR TO HOLD THE PARAMETER UPDATES (NPx1)
	qvec - BACKUP COPY OF PARAMETER VECTOR para (NPx1)
	gstep - REAL*8 VECTOR IN GAUSS-NEWTON STEP (NPx1) (PASSED FROM ITER1)
	infmat - REAL*8 INFORMATION MATRIX (NPxNP)(PASSED FROM ITER1)
	invinf - REAL*8 INVERSE OF INFORMATION MATRIX (NPxNP)
  */
  double det;
  int i;

  if(show_flag) printf("Iteration %d\n",num_iter);
  dmcpy_(invinf,infmat,fit_puse,fit_puse);
  dminv_(invinf,fit_puse,&det,gvec,qvec);
  if(fabs(det)<1e-20){
    /* Non-singular information matrix encountered (determinant<1E-20)*/
    /*c set all s.e. to -1*/
    for(i=0;i<fit_npar;i++){
      fit_semat[i] = -1;
    }
    return 1;
  }
  dmmul_(gvec,invinf,gstep,fit_puse,fit_puse,1);
  /*
    SAVE PRESENT PARAMETER VECTOR AS BASE FOR UPDATES AND
    TO EVALUATE PERCENTAGE CHANGE.
  */
  for(i=0;i<fit_puse;i++){
    qvec[i] = para[fit_fixmat[i] - 1];
  }
  return 0;
}
