/* Bld version multilinear regression */
/* Copyright (C) 2000-2018 Hiroshi Watabe watabe@cyric.tohoku.ac.jp */
/* $Id: bld_mullin.c,v 2fcc4ac67d16 2018/05/14 22:42:12 watabe $ */
#include "bld.h"

PyArrayObject *mullin(PyArrayObject *dep_y_ar, PyArrayObject *indep_x_ar, PyArrayObject *wt_ar, int weight,int cst)
{
  int n_indepy,n_var,i;
  float *para,*dep_y,*indep_x,*wt,*cormat;
  double *p_se,*fit;
  int df[2];
  double ss,secu,*b,*work;
  PyArrayObject *result;
  npy_intp dimensions[1];
  char bweight;

  import_array();	
  n_indepy = dep_y_ar->dimensions[0];
  if((dep_y = array2float(dep_y_ar))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable dep_y");
    return NULL;
  }
  n_var = indep_x_ar->dimensions[0];
  if(weight){
    if((wt = array2float(wt_ar))==NULL){
      PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable wt");
      return NULL;
    }
    bweight = 1;
  }
  else{wt = NULL;bweight = 0; }

  para = (float *)malloc(sizeof(float)*n_var);
  p_se = (double *)malloc(sizeof(double)*n_var);
  fit =  (double *)malloc(sizeof(double)*n_indepy);
  b = (double *)malloc(sizeof(double)*n_var*n_var);
  work = (double *)malloc(sizeof(double)*n_var);
  cormat = (float *)malloc(sizeof(float)*n_var*n_var);
  if((para==NULL)||(p_se==NULL)||(fit==NULL)||(b==NULL)||(work==NULL)||(cormat==NULL)) {
    PyErr_SetString(PyExc_MemoryError, "Cannot allocate memory");
    return NULL;
  }
  if((indep_x = array2float2d(indep_x_ar))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable indep_x");
    return NULL;
  }

  if(mullin_(n_indepy,n_var,indep_x,dep_y,para,p_se,wt,&bweight,fit,&ss,&secu,b,work,cormat)){
    PyErr_SetString(PyExc_ValueError, "Non-singular information matrix detected");
    return NULL;
  }

  if(weight){
    printf("Weighted regression\n");
  }
  if(show_flag){
    printf("Parameters and error estimates :\n");
    for(i=0;i<n_var;i++){
      printf("No. = %d Coeff. = %g S.E. = %g\n",i + 1,para[i],p_se[i]);
    }
  }
  /* statistics report */
  df[0] = n_var;
  if(cst){ df[0] = n_var - 1;}
  df[1] = n_indepy - n_var;
  stat_report(dep_y,1,n_indepy,wt,weight,cst,ss,secu,df);
  
  dimensions[0] = 1 + n_var + n_var + 7 + n_indepy + n_var*n_var;/* n_var,para,se,fit,ss,secu,cormat report_mat*/
  
  result = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_FLOAT);
  (*(float *)(result->data)) = (float)n_var;
  for(i=0;i<n_var;i++){
    (*(float *)(result->data + (i + 1)*result->strides[0])) = (float)para[i];
  }
  for(i=0;i<n_var;i++){
    (*(float *)(result->data + (1 + i + n_var)*result->strides[0])) = (float)p_se[i];
  }
  for(i=0;i<7;i++){
    (*(float *)(result->data + (i + 1 + n_var*2)*result->strides[0])) = (float)report_mat[i];
  }
  for(i=0;i<n_var*n_var;i++){
    (*(float *)(result->data + (i + 1 + 7 + 2*n_var)*result->strides[0])) = (float)cormat[i];
  }
  for(i=0;i<n_indepy;i++){
    (*(float *)(result->data + (i + 1 + 7 + n_var*2 + n_var*n_var)*result->strides[0])) = (float)fit[i];
  }

  free(cormat);
  free(dep_y);
  free(indep_x);
  free(fit);
  free(wt);
  free(para);
  free(p_se);
  free(b);
  free(work);
  return (PyArrayObject *)PyArray_Return(result);
}
 
