/* polinominal fitting BLD version */
/* Copyright (C) 2000-2008 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
#include "bld.h"

float *prepare_polfit_x(PyArrayObject*,int);

PyArrayObject *polfit(int deg_pol, PyArrayObject *dep_y_ar, PyArrayObject *dep_x_ar, PyArrayObject *wt_ar, int weight)
{
  int n_depy,n_var,i,j;
  float *para,*dep_y,*dep_x,*wt,*cormat;
  double selast;
  double *fit,*p_se;
  int df[2];
  double ss,secu,*b,*work;
  PyArrayObject *result;
  npy_intp dimensions[1];
  char bweight;
  int nord;

  import_array();	
  n_var = deg_pol + 1;
  selast = 1.0e30;
  nord = 1;
  if(!PyArray_Check(dep_y_ar) || !PyArray_Check(dep_x_ar)){
    PyErr_SetString(PyExc_TypeError, "Need a Numeric.array for x and y arguments");
    return NULL;
  }
  n_depy = dep_y_ar->dimensions[0];
  if((dep_y = array2float(dep_y_ar))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to read variable dep_y");
    return NULL;
  }
  if(weight){
    if((wt = array2float(wt_ar))==NULL){
      PyErr_SetString(PyExc_MemoryError, "Failed to read variable wt");
      return NULL;
    }
    bweight = 1;
  }
  else{wt = NULL;bweight = 0; }

  para = (float *)malloc(sizeof(float)*n_var);
  p_se = (double *)malloc(sizeof(double)*n_var);
  fit =  (double *)malloc(sizeof(double)*n_depy);
  b = (double *)malloc(sizeof(double)*n_var*n_var);
  work = (double *)malloc(sizeof(double)*n_var);
  cormat = (float *)malloc(sizeof(float)*n_var*n_var);
  if((para==NULL)||(p_se==NULL)||(fit==NULL)||(b==NULL)||(work==NULL)||(cormat==NULL)) {
    PyErr_SetString(PyExc_MemoryError, "cannot allocate memory");
    return NULL;
}
  if((dep_x = prepare_polfit_x(dep_x_ar,n_var))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable dep_x");
    return NULL;
  }

  nord = 1;
  while(nord<n_var){
    nord++;
    mullin_(n_depy,nord,dep_x,dep_y,para,p_se,wt,&bweight,fit,&ss,&secu,b,work,cormat);
    if(ss<0.0) {
      PyErr_SetString(PyExc_ValueError, "Non-singular information matrix detected");
      return NULL;
    }
    if(show_flag)printf("Polynominal regression of degree %d\n",nord - 1);
    if(selast==-1) break;
    if(secu<selast){/* improved */
      selast = secu;
      for(j=0;j<=nord-1;j++){
	if(show_flag)printf("Order  %d Coeff. = %g S.E. = %g\n",j,para[j],p_se[j]);
      }
    }
    else{
      /*
C  THERE WAS NO IMPROVEMENT FOR THIS ORDER POLYNOMIAL.  THE
C	ANALYSIS FOR THE PREVIOUS ORDER WILL BE REPEATED.
      */
      if(show_flag) printf("No improvement\n");
      selast = -1;
      nord -= 2;
    }
  }

  if(weight){
    printf("Weighted regression\n");
  }
  if(show_flag){
	printf("Parameters and error estimates :\n");
	  for(i=0;i<n_var;i++){
	    printf("No. = %d Coeff. = %g S.E. = %g\n",i + 1,para[i],p_se[i]);
	  }
	}
  /* statistics report */
  if(nord==1){
    df[0] = 1;
    df[1] = n_depy - 1;
    stat_report(dep_y,1,n_depy,wt,weight,0,ss,secu,df);
  }
  else{
    df[0] = nord - 1;
    df[1] = n_depy - nord;
    stat_report(dep_y,1,n_depy,wt,weight,1,ss,secu,df);
  }
  
  n_var = nord;
  dimensions[0] = 1 + n_var + n_var + n_depy + 7 + n_var*n_var;/* n_var,para,se,fit,report_mat,cormat */
  
  result = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_FLOAT);
  (*(float *)(result->data)) = (float)n_var;
  for(i=0;i<n_var;i++){
    (*(float *)(result->data +(i + 1)*result->strides[0])) = (float)para[i];
  }
  for(i=0;i<n_var;i++){
    (*(float *)(result->data + (i + 1 + n_var)*result->strides[0])) = (float)p_se[i];
  }
  for(i=0;i<7;i++){
    (*(float *)(result->data + (i + 1 + 2*n_var)*result->strides[0])) = (float)report_mat[i];
  }
  for(i=0;i<n_var*n_var;i++){
    (*(float *)(result->data + (i + 1 + 7 + 2*n_var)*result->strides[0])) = (float)cormat[i];
  }
  for(i=0;i<n_depy;i++){
    (*(float *)(result->data + (i + 1 + 7 + 2*n_var + n_var*n_var)*result->strides[0])) = (float)fit[i];
  }

  free(cormat);
  free(dep_y);
  free(dep_x);
  free(fit);
  free(p_se);
  free(b);
  free(work);
  free(para);
  if(wt!=NULL){free(wt);}
  return (PyArrayObject *)PyArray_Return(result);
}
 
float *prepare_polfit_x(PyArrayObject *array, int n_var)
{
  int ndim,i,j;
  float *fmat;
  ndim = array->dimensions[0];
  fmat = (float *)malloc(sizeof(float)*ndim*n_var);
  if(fmat==NULL){
    PyErr_SetString(PyExc_MemoryError, "cannot allocate memory");
    return NULL;
  }
  for(i=0;i<ndim;i++){
    fmat[i] = 1.0; /* constant */
  }
  for(i=0;i<ndim;i++){
    if(array->descr->type_num == PyArray_FLOAT){
      fmat[i + ndim] = (float)(*(float *)(array->data + i*array->strides[0]));
    }
    else{
      if(array->descr->type_num == PyArray_DOUBLE){
	fmat[i + ndim] = (float)(*(double *)(array->data + i*array->strides[0]));
      }
      else{/* no float or double */
	PyErr_SetString(PyExc_TypeError, "Please give float or double array");
	return NULL;
      }
    }
  }
  if(n_var<2) return(fmat);

  for(j=2;j<n_var;j++){
    for(i=0;i<ndim;i++){
      fmat[i + ndim*j] = (float)pow((double)fmat[i + ndim],(double)j);
    }
  }
  return(fmat);
}
