/* SUPERVISE AND PERFORM A SPLINE FIT */
/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
#include "bld.h"

PyArrayObject *spline(PyArrayObject *x_ar,PyArrayObject *y_ar,PyArrayObject *p_ar)
{
  double epsln;
  float *xdat,*ydat,*pdat;
  int n_y,n_yp,n1,i,j;
  float *hmat,*qmat,*amat,*bmat,*cmat,*dmat,*yp;
  double hph,dltdlt,omega,est,W,T,PX1,PX2,PX1PX2,DDYP,DLTAPA;
  PyArrayObject *result;
  npy_intp dimensions[1];

  epsln = 1.0e-20;
  import_array();	

  n_y = y_ar->dimensions[0];
  n_yp = p_ar->dimensions[0];
  if((ydat = array2float(y_ar))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable y in spline fit");
    return NULL;
  }
  if((xdat = array2float(x_ar))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable x in spline fit");
    return NULL;
  }
  if((pdat = array2float(p_ar))==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to prepare variable p in spline fit");
    return NULL;
  }
 
/*   A IS THE INITIAL APPROXIMATION TO Y''*/

  hmat = (float *)malloc(n_y*sizeof(float));
  qmat = (float *)malloc(n_y*sizeof(float));
  amat = (float *)malloc(n_y*sizeof(float));
  bmat = (float *)malloc(n_y*sizeof(float));
  cmat = (float *)malloc(n_y*sizeof(float));
  dmat = (float *)malloc(n_y*sizeof(float));
  yp = (float *)malloc(n_yp*sizeof(float));
  n1 = n_y - 1;
  for(i=0;i<n1;i++){
    hmat[i] = xdat[i + 1] - xdat[i];
    qmat[i] = (ydat[i + 1] - ydat[i])/hmat[i];
  }
  for(i=1;i<n1;i++){
    hph  = hmat[i-1] + hmat[i];
    bmat[i] = (float)(.5*hmat[i - 1]/hph);
    dltdlt = (qmat[i] - qmat[i - 1])/hph;
    amat[i] = (float)(dltdlt + dltdlt);
    cmat[i] = (float)(3.*dltdlt);
  }

  /*   SOLVE FOR Y''*/

  amat[0] = 0.;
  amat[n_y - 1] =0.;
  omega = 1.0717968;
  do {
    est = 0.;
    for(i=1;i<n1;i++){
      W = (cmat[i] - bmat[i]*amat[i - 1] - (.5 - bmat[i])*amat[i + 1] - amat[i])*omega;
      T = fabs(W);
      if(T>est){
	est = T;
      }
      amat[i] = (float)(amat[i] + W);
    }
  }
  while(est<epsln);

  /*  SOLVE FOR Y Y' AND Y'' AT EACH VALUE IN P*/
  for(i=0;i<n1;i++){
    dmat[i] = (amat[i+1] - amat[i])/hmat[i];
  }

  for(j=0;j<n_yp;j++){
    if(xdat[0]>pdat[j]){
      yp[j] = ydat[0];
      continue;
    }
    for(i=1;i<n_y;i++){
      if(pdat[j]<xdat[i]) break;
    }
    if(i==n_y){
      yp[j] = ydat[n_y - 1];
      continue;
    }
    i = i - 1;
    PX1=pdat[j] - xdat[i];
    PX2=pdat[j] - xdat[i + 1];
    PX1PX2=PX1*PX2;
    DDYP=amat[i] + PX1*dmat[i];//	!SECOND DERIVATIVE
    DLTAPA=(amat[i] + amat[i + 1] + DDYP) / 6.;
    yp[j] = (float)(ydat[i] + PX1*qmat[i] + PX1PX2*DLTAPA);
  }
  dimensions[0] = n_yp;
  result = (PyArrayObject *)PyArray_SimpleNew(1,dimensions,PyArray_FLOAT);
  for(i=0;i<n_yp;i++){
    (*(float *)(result->data + i*result->strides[0])) = (float)yp[i];
  }
  free(hmat);
  free(qmat);
  free(amat);
  free(bmat);
  free(cmat);
  free(dmat);
  return (PyArrayObject *)PyArray_Return(result);
}
