/* sub iteration for BLD fitting */
/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* $Id$ */
#include <math.h>
#include "bld.h"

/*
C  SUBIT(B,G,Q,V,iptr) - THIS LOGICAL*1 FUNCTION HANDLES
C	THE SUB-ITERATIONS FOR NON-LINEAR REGRESSION.  IT APPLIES THE MAIN
C	REGRESSION STEP TO THE PARAMETER VECTOR AND CALCULATES THE SUM OF
C	SQUARES.  IF THERE HAS BEEN AN IMPROVEMENT SINCE THE LAST ITERATION
C	IT QUITS.  IF NOT, IT HALVES THE STEP UNTIL AN IMPROVEMENT HAS OCCURRED
C	OR UNIL 'IHFMAX' STEP HALVINGS HAVE OCCURRED.  MUCH OF THE CODE
C	HAS BEEN COMMENTED OUT.  IT CONTAINED THE CODE NECESSARY TO
C	IMPLEMENT A MARQUARDT STRATEGY OF SUB-ITERATIONS  (ADDING A DIAGONAL
C	MATRIX TO THE INFORMATION MATRIX BEFORE INVERTING AND UPDATING
C	THE PARAMETER ESTIMATES).  THE ROUTINE RETURNS TRUE IF A FUNCTION
C	EVALUATION ERROR OCCURS.
C
C   ARGUMENTS:
C	NP - INTEGER NUMBER OF PARAMETERS
C	B - REAL*8 VECTOR OF PARAMETERS
C	G - REAL*8 VECTOR HOLDING THE PARAMETER UPDATES (NPx1) (INPUT)
C	Q - REAL*8 WORK VECTOR HOLDING INITIAL PARAMETER VECTOR (NPx1) (INPUT)
C	V - REAL*8 WORK VECTOR FOR FDG PARAMETER ADJUSTMENT(NPx1) 
*/
int subit(double *parm,double *gvec,double *qvec)
{
  int k4 = 0;
  int i;
  double s1;
  double d1;
  /*
C  ADD PARAMETER STEP (gvec) TO PREVIOUS PARAMETER VALUES (qvec)
  */

  while(1){
    for(i=0;i<fit_puse;i++){
      parm[fit_fixmat[i] - 1] = qvec[i] + gvec[i];
    }
    /*  COMPUTE THE NEW SUM OF SQUARES*/
    if(eval_func(parm,fit_func)){
      return(1);
    }
    sumsq(&s1);
    /*  OUTPUT MESSAGE*/
    if(show_flag) printf("Sub-iteration : %d Sum of squares = %lg\n",k4,s1);
    k4 += 1;
    if((s1<cur_ss)||(k4>fit_subiterations)) break;
    /*  HALVE THE STEP SIZE*/
    d1 = 0.5;
    i = 1;
    dmmulk_(gvec,gvec,d1,i,fit_puse);
  } /* 1460 */
  /*  END OF SUB-ITERATIONS - SAVE NEW SUM OF SQUARES*/
  cur_ss = s1;
  return(0);
}
