/* ROUTINE TO TEST FOR CONVERGENCE IN NON-LINEAR REGRESSION for BLD fitting */
/* $Id */
/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
#include <math.h>
#include "bld.h"

/*
C  TESTCG(B,Q) - THIS LOGICAL*1 FUNCTION IS RESPONSIBLE FOR TESTING
C	FOR CONVERGENCE IN NON-LINEAR REGRESSION.  THE REGRESSION IS CONSIDERED
C	TO CONVERGE IF ALL PARAMETERS HAVE CHANGED BY LESS THAN ONE TENTH
C	OF ONE PERCENT FROM THE PREVIOUS ITERATION.  IF SO, THE LOGICAL
C	FLAG CONV IN COMMON /FIT/ IS SET TO TRUE.  TESTCG ALWAYS RETURNS
C	AS FALSE SO THAT IT ACTUALLY NO LONGER NEEDS TO BE A LOGICAL FUNCTION.
C
C  ARGUMENTS:
C   NP - NUMBER OF PARAMETERS
C   B - REAL*8 VECTOR OF UPDATED PARAMETERS
C   Q - REAL*8 VECTOR OF PREVIOUS PARAMETERS
*/
int testcg(double *parm,double *qvec){
  unsigned char temp;
  int i,ii;
  double s3;

  /*  PRINT OUT THE NEW PARAMETERS AND THEIR ABSOLUTE AND PERCENTAGE CHANGE.
C	CONVERGENCE HAS BEEN ACHIEVED IF NONE OF THE PARAMETERS HAS
C	CHANGED BY MORE THAN ONE TENTH OF ONE PERCENT.
  */
  if(show_flag)printf("End of sub-iterations\n");
  temp = conv;
  conv = 1;
  /*  LOOP THROUGH PARAMETERS*/
  for(i=0;i<fit_puse;i++){
    ii = fit_fixmat[i] - 1;
    if(parm[ii]==0){
      s3 = -qvec[i]*1.0e9;
    }
    else{
      s3 = (1.0 - qvec[i]/parm[ii])*100.0;/*	!PERCENT CHANGE*/
    }
    /*  PRINT OUT FOR THIS PARAMETER*/
    if(show_flag){
      printf(" Para %d = %lg Change = %lg %% Chg = %lg\n",ii + 1,parm[ii],parm[ii] - qvec[i],s3);
    }
    if(fabs(s3)>fit_converge) conv = temp; /*LARGE CHANGE==>NO CONVERGENCE*/
  }
  ktlx = 8;
  /*  print out message in brief mode*/
  if(show_flag) printf("Iteration %d Sum of squares= %lg\n",num_iter,cur_ss);
  return 0;
}
