/* several small utilities for BLD program */
/* Copyright (C) 2000-2019 Hiroshi Watabe watabe@cyric.tohoku.ac.jp */
/* $Id: bld_util.c,v f68803db16f4 2019/07/28 23:00:14 watabe $ */
#include <math.h>
#include "bld.h"

double report_mat[7];

int *array2int(PyArrayObject *array)
{
  int ndim,i;
  int *fmat;
  ndim = array->dimensions[0];
  fmat = (int *)malloc(sizeof(int)*ndim);
  if(fmat==NULL){
    fprintf(stderr,"cannot allocate memory in array2int!! %d\n",ndim);
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory in array2int");
    return NULL;
  }
  for(i=0;i<ndim;i++){
    if(array->descr->type_num == PyArray_FLOAT){
      fmat[i] = (int)(*(float *)(array->data + i*array->strides[0]));
    }
    else{
      if(array->descr->type_num == PyArray_DOUBLE){
	fmat[i] = (int)(*(double *)(array->data + i*array->strides[0]));
      }
      else{/* no float or double */
	if(array->descr->type_num == PyArray_SHORT){
	  fmat[i] = (int)(*(short *)(array->data + i*array->strides[0]));
	}
	else{
	  if(array->descr->type_num == PyArray_SHORT){
	    fmat[i] = (int)(*(short *)(array->data + i*array->strides[0]));
	  }
	  else{
	    if(array->descr->type_num == PyArray_INT){
	      fmat[i] = (int)(*(int *)(array->data + i*array->strides[0]));
	    }
	    else{
	      if(array->descr->type_num == PyArray_LONG){
		fmat[i] = (int)(*(long *)(array->data + i*array->strides[0]));
	      }
	      else{
		if(array->descr->type_num == PyArray_UBYTE){
		  fmat[i] = (int)(*(unsigned char *)(array->data + i*array->strides[0]));
		}
		else{
		  if(array->descr->type_num == PyArray_INT64){
		    fmat[i] = (int)(*(long long *)(array->data + i*array->strides[0]));
		  }
		  else{
		    PyErr_SetString(PyExc_ValueError, "Failed to convert to INT");
		    return NULL;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}
float *array2float(PyArrayObject *array)
{
  int ndim,i;
  float *fmat;
  ndim = array->dimensions[0];
  fmat = (float *)malloc(sizeof(float)*ndim);
  if(fmat==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory in array2float");
    return NULL;
  }
  for(i=0;i<ndim;i++){
    if(array->descr->type_num == PyArray_FLOAT){
      fmat[i] = (float)(*(float *)(array->data + i*array->strides[0]));
    }
    else{
      if(array->descr->type_num == PyArray_DOUBLE){
	fmat[i] = (float)(*(double *)(array->data + i*array->strides[0]));
      }
      else{/* no float or double */
	if(array->descr->type_num == PyArray_SHORT){
	  fmat[i] = (float)(*(short *)(array->data + i*array->strides[0]));
	}
	else{
	  if(array->descr->type_num == PyArray_INT){
	    fmat[i] = (float)(*(int *)(array->data + i*array->strides[0]));
	  }
	  else{
	    if(array->descr->type_num == PyArray_LONG){
	      fmat[i] = (float)(*(long *)(array->data + i*array->strides[0]));
	    }
	    else{
	      if(array->descr->type_num == PyArray_INT64){
		fmat[i] = (float)(*(long long *)(array->data + i*array->strides[0]));
	      }
	      else{
		if(array->descr->type_num == PyArray_UBYTE){
		  fmat[i] = (float)(*(unsigned char *)(array->data + i*array->strides[0]));
		}
		else{
		  PyErr_SetString(PyExc_ValueError, "Failed to convert to FLOAT");
		  return NULL;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}
double *array2double(PyArrayObject *array)
{
  int ndim,i;
  double *fmat;
  ndim = array->dimensions[0];
  fmat = (double *)malloc(sizeof(double)*ndim);
  if(fmat==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory in array2double");
    return NULL;
  }
  for(i=0;i<ndim;i++){
    if(array->descr->type_num == PyArray_FLOAT){
      fmat[i] = (double)(*(float *)(array->data + i*array->strides[0]));
    }
    else{
      if(array->descr->type_num == PyArray_DOUBLE){
	fmat[i] = (double)(*(double *)(array->data + i*array->strides[0]));
      }
      else{/* no float or double */
	if(array->descr->type_num == PyArray_SHORT){
	  fmat[i] = (double)(*(short *)(array->data + i*array->strides[0]));
	}
	else{
	  if(array->descr->type_num == PyArray_SHORT){
	    fmat[i] = (double)(*(short *)(array->data + i*array->strides[0]));
	  }
	  else{
	    if(array->descr->type_num == PyArray_INT){
	      fmat[i] = (double)(*(int *)(array->data + i*array->strides[0]));
	    }
	    else{
	      if(array->descr->type_num == PyArray_LONG){
		fmat[i] = (double)(*(long *)(array->data + i*array->strides[0]));
	      }
	      else{
		if(array->descr->type_num == PyArray_UBYTE){
		  fmat[i] = (double)(*(unsigned char *)(array->data + i*array->strides[0]));
		}
		else{
		  PyErr_SetString(PyExc_ValueError, "Failed to convert to DOUBLE");
		  return NULL;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}
/* 2dimensional */
float *array2float2d(PyArrayObject *array)
{
  int ndim,pardim,i,j;
  float *fmat;
  pardim = array->dimensions[1];
  ndim = array->dimensions[0];
  fmat = (float *)malloc(sizeof(float)*ndim*pardim);
  if(fmat==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory in array2float2d");
    return NULL;
  }
  for(i=0;i<ndim;i++){
    for(j=0;j<pardim;j++){
      if(array->descr->type_num == PyArray_FLOAT){
	fmat[i*pardim + j] = (float)(*(float *)(array->data + j*array->strides[1] + i*array->strides[0]));
      }
      else{
	if(array->descr->type_num == PyArray_DOUBLE){
	  fmat[i*pardim + j] = (float)(*(double *)(array->data + j*array->strides[1] + i*array->strides[0]));
	}
	else{/* no float or double */
	  if(array->descr->type_num == PyArray_SHORT){
	    fmat[i*pardim + j] = (float)(*(short *)(array->data + j*array->strides[1] + i*array->strides[0]));
	  }
	  else{
	    if(array->descr->type_num == PyArray_INT){
	      fmat[i*pardim + j] = (float)(*(int *)(array->data + j*array->strides[1] + i*array->strides[0]));
	    }
	    else{
	      if(array->descr->type_num == PyArray_LONG){
		fmat[i*pardim + j] = (float)(*(long *)(array->data + j*array->strides[1] + i*array->strides[0]));
	      }
	      else{
		if(array->descr->type_num == PyArray_UBYTE){
		  fmat[i*pardim + j] = (float)(*(unsigned char *)(array->data + j*array->strides[1] + i*array->strides[0]));
		}
		else{
		  PyErr_SetString(PyExc_ValueError, "Failed to convert to FLOAT 2D");
		  return NULL;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}
/* 3 dimensional */
float *array2float3d(PyArrayObject *array)
{
  int ndim,pardim,xdim,i,j,k;
  float *fmat;
  pardim = array->dimensions[1];
  ndim = array->dimensions[0];
  xdim = array->dimensions[2];
  fmat = (float *)malloc(sizeof(float)*ndim*pardim*xdim);
  if(fmat==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory in array2float3d");
    return NULL;
  }
  for(i=0;i<ndim;i++){
    for(j=0;j<pardim;j++){
      for(k=0;k<xdim;k++){
	if(array->descr->type_num == PyArray_FLOAT){
	  fmat[i*pardim*xdim + j*xdim + k] = (float)(*(float *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2]));
	}
	else{
	  if(array->descr->type_num == PyArray_DOUBLE){
	    fmat[i*pardim*xdim + j*xdim + k] = (float)(*(double *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2]));
	  }
	  else{/* no float or double */
	    if(array->descr->type_num == PyArray_SHORT){
	      fmat[i*pardim*xdim + j*xdim + k] = (float)(*(short *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2]));
	    }
	    else{
	      if(array->descr->type_num == PyArray_INT){
		fmat[i*pardim*xdim + j*xdim + k] = (float)(*(int *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2]));
	      }
	      else{
		if(array->descr->type_num == PyArray_LONG){
		  fmat[i*pardim*xdim + j*xdim + k] = (float)(*(long *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2]));
		}
		else{
		  if(array->descr->type_num == PyArray_UBYTE){
		    fmat[i*pardim*xdim + j*xdim + k] = (float)(*(unsigned char *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2]));
		  }
		  else{
		    PyErr_SetString(PyExc_ValueError, "Failed to convert to FLOAT 3D");
		    return NULL;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}

/* 4 dimensional */
float *array2float4d(PyArrayObject *array)
{
  int tdim,zdim,ydim,xdim,i,j,k,l;
  float *fmat;
  tdim = array->dimensions[0];
  zdim = array->dimensions[1];
  ydim = array->dimensions[2];
  xdim = array->dimensions[3];
  fmat = (float *)malloc((long)(sizeof(float)*tdim*zdim*ydim*xdim));
  if(fmat==NULL){
    PyErr_SetString(PyExc_MemoryError, "Failed to allocate memory in array2float4d");
    return NULL;
  }
  for(i=0;i<tdim;i++){
    for(j=0;j<zdim;j++){
      for(k=0;k<ydim;k++){
	for(l=0;l<xdim;l++){
	  if(array->descr->type_num == PyArray_FLOAT){
	    fmat[i*ydim*zdim*xdim + j*ydim*xdim + k*xdim + l] = (float)(*(float *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2] + l*array->strides[3]));
	  }
	  else{
	    if(array->descr->type_num == PyArray_DOUBLE){
	      fmat[i*ydim*zdim*xdim + j*ydim*xdim + k*xdim + l] = (float)(*(double *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2] + l*array->strides[3]));
	    }
	    else{/* no float or double */
	      if(array->descr->type_num == PyArray_SHORT){
		fmat[i*ydim*zdim*xdim + j*ydim*xdim + k*xdim + l] = (float)(*(short *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2] + l*array->strides[3]));
	      }
	      else{
		if(array->descr->type_num == PyArray_INT){
		  fmat[i*ydim*zdim*xdim + j*ydim*xdim + k*xdim + l] = (float)(*(int *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2] + l*array->strides[3]));
		}
		else{
		  if(array->descr->type_num == PyArray_LONG){
		    fmat[i*ydim*zdim*xdim + j*ydim*xdim + k*xdim + l] = (float)(*(long *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2] + l*array->strides[3]));
		  }
		  else{
		    if(array->descr->type_num == PyArray_UBYTE){
		      fmat[i*ydim*zdim*xdim + j*ydim*xdim + k*xdim + l] = (float)(*(unsigned char *)(array->data + j*array->strides[1] + i*array->strides[0] + k*array->strides[2] + l*array->strides[3]));
		    }
		    else{
		      PyErr_SetString(PyExc_ValueError, "Failed to convert to FLOAT 4D");
		      return NULL;
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}

double *array2double2d(PyArrayObject *array)
{
  int ndim,pardim,i,j;
  double *fmat;
  pardim = array->dimensions[1];
  ndim = array->dimensions[0];
  fmat = (double *)malloc(sizeof(double)*ndim*pardim);
  if(fmat==NULL){
    fprintf(stderr,"cannot allocate memory in array2double2d!! pardim %d ndim %d\n",pardim,ndim);
    return NULL;
  }
  for(i=0;i<ndim;i++){
    for(j=0;j<pardim;j++){
      if(array->descr->type_num == PyArray_FLOAT){
	fmat[i*pardim + j] = (double)(*(float *)(array->data + j*array->strides[1] + i*array->strides[0]));
      }
      else{
	if(array->descr->type_num == PyArray_DOUBLE){
	  fmat[i*pardim + j] = (double)(*(double *)(array->data + j*array->strides[1] + i*array->strides[0]));
	}
	else{/* no float or double */
	  if(array->descr->type_num == PyArray_SHORT){
	    fmat[i*pardim + j] = (double)(*(short *)(array->data + j*array->strides[1] + i*array->strides[0]));
	  }
	  else{
	    if(array->descr->type_num == PyArray_INT){
	      fmat[i*pardim + j] = (double)(*(int *)(array->data + j*array->strides[1] + i*array->strides[0]));
	    }
	    else{
	      if(array->descr->type_num == PyArray_LONG){
		fmat[i*pardim + j] = (double)(*(long *)(array->data + j*array->strides[1] + i*array->strides[0]));
	      }
	      else{
		if(array->descr->type_num == PyArray_UBYTE){
		  fmat[i*pardim + j] = (double)(*(unsigned char *)(array->data + j*array->strides[1] + i*array->strides[0]));
		}
		else{
		  PyErr_SetString(PyExc_ValueError, "Failed to convert to DOUBLE 2D");
		return NULL;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return(fmat);
}

int float2array2d(float *fmat,PyArrayObject *array)
{
  int ndim,mdim,i,j;
  ndim = array->dimensions[0];
  mdim = array->dimensions[1];

  for(i=0;i<ndim;i++){
    for(j=0;j<mdim;j++){
      if(array->descr->type_num == PyArray_FLOAT){
	(*(float *)(array->data + i*array->strides[0] + j*array->strides[1])) = fmat[i*mdim +j];
      }
      else{
	if(array->descr->type_num == PyArray_DOUBLE){
	  (*(double *)(array->data + i*array->strides[0] + j*array->strides[1])) = (double)fmat[i*mdim +j];
	}
	else{/* no float or double */
	  if(array->descr->type_num == PyArray_SHORT){
	    (*(short *)(array->data + i*array->strides[0] + j*array->strides[1])) = (short)fmat[i*mdim +j];
	  }
	  else{
	    if(array->descr->type_num == PyArray_INT){
	      (*(int *)(array->data + i*array->strides[0] + j*array->strides[1])) = (int)fmat[i*mdim +j];
	    }
	    else{
	      if(array->descr->type_num == PyArray_LONG){
		(*(long *)(array->data + i*array->strides[0] + j*array->strides[1])) = (long)fmat[i*mdim +j];
	      }
	      else{
		if(array->descr->type_num == PyArray_UBYTE){
		  (*(unsigned char *)(array->data + i*array->strides[0] + j*array->strides[1])) = (long)fmat[i*mdim +j];
		}
		else{
		  return 1;
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return 0;
}

int float2array3d(float *fmat,PyArrayObject *array)
{
  int ndim,mdim,xdim,i,j,k;
  ndim = array->dimensions[0];
  mdim = array->dimensions[1];
  xdim = array->dimensions[2];

  for(i=0;i<ndim;i++){
    for(j=0;j<mdim;j++){
      for(k=0;k<xdim;k++){
	if(array->descr->type_num == PyArray_FLOAT){
	  (*(float *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2])) = fmat[i*mdim*xdim +j*xdim + k];
	}
	else{
	  if(array->descr->type_num == PyArray_DOUBLE){
	    (*(double *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2])) = (double)fmat[i*mdim*xdim +j*xdim + k];
	  }
	  else{/* no float or double */
	    if(array->descr->type_num == PyArray_SHORT){
	      (*(short *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2])) = (short)fmat[i*mdim*xdim +j*xdim + k];
	    }
	    else{
	      if(array->descr->type_num == PyArray_INT){
		(*(int *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2])) = (int)fmat[i*mdim*xdim +j*xdim + k];
	      }
	      else{
		if(array->descr->type_num == PyArray_LONG){
		  (*(long *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2])) = (long)fmat[i*mdim*xdim +j*xdim + k];
		}
		else{
		  if(array->descr->type_num == PyArray_UBYTE){
		    (*(unsigned char *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2])) = (long)fmat[i*mdim*xdim +j*xdim + k];
		  }
		  else{
		    return 1;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return 0;
}

int float2array4d(float *fmat,PyArrayObject *array)
{
  int xdim,ydim,zdim,tdim,i,j,k,l;
  tdim = array->dimensions[0];
  zdim = array->dimensions[1];
  ydim = array->dimensions[2];
  xdim = array->dimensions[3];

  for(i=0;i<tdim;i++){
    for(j=0;j<zdim;j++){
      for(k=0;k<ydim;k++){
	for(l=0;l<xdim;l++){
	  if(array->descr->type_num == PyArray_FLOAT){
	    (*(float *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2] + l*array->strides[3])) = fmat[i*zdim*ydim*xdim +j*ydim*xdim + k*xdim + l];
	  }
	  else{
	    if(array->descr->type_num == PyArray_DOUBLE){
	      (*(double *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2] + l*array->strides[3])) = (double)fmat[i*zdim*ydim*xdim +j*ydim*xdim + k*xdim + l];
	    }
	    else{/* no float or double */
	      if(array->descr->type_num == PyArray_SHORT){
		(*(short *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2] + l*array->strides[3])) = (short)fmat[i*zdim*ydim*xdim +j*ydim*xdim + k*xdim + l];
	      }
	      else{
		if(array->descr->type_num == PyArray_INT){
		  (*(int *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2] + l*array->strides[3])) = (int)fmat[i*zdim*ydim*xdim +j*ydim*xdim + k*xdim + l];
		}
		else{
		  if(array->descr->type_num == PyArray_LONG){
		    (*(long *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2] + l*array->strides[3])) = (long)fmat[i*zdim*ydim*xdim +j*ydim*xdim + k*xdim + l];
		  }
		  else{
		    if(array->descr->type_num == PyArray_UBYTE){
		      (*(unsigned char *)(array->data + i*array->strides[0] + j*array->strides[1] + k*array->strides[2] + l*array->strides[3])) = (long)fmat[i*zdim*ydim*xdim +j*ydim*xdim + k*xdim + l];
		    }
		    else{
		      return 1;
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  return 0;
}

int float2array1d(float *fmat,PyArrayObject *array)
{
  int ndim,i;
  ndim = array->dimensions[0];
  for(i=0;i<ndim;i++){
    if(array->descr->type_num == PyArray_FLOAT){
      (*(float *)(array->data + i*array->strides[0])) = fmat[i];
    }
    else{
      if(array->descr->type_num == PyArray_DOUBLE){
	(*(double *)(array->data + i*array->strides[0])) = (double)fmat[i];
      }
      else{/* no float or double */
	return 1;
      }
    }
  }
  return 0;
}

int double2array2d(double *dmat,PyArrayObject *array)
{
  int ndim,mdim,i,j;
  ndim = array->dimensions[0];
  mdim = array->dimensions[1];

  for(j=0;j<mdim;j++){
    for(i=0;i<ndim;i++){
      if(array->descr->type_num == PyArray_FLOAT){
	(*(float *)(array->data + i*array->strides[0] + j*array->strides[1])) = (float)dmat[i + ndim*j];
      }
      else{
	if(array->descr->type_num == PyArray_DOUBLE){
	  (*(double *)(array->data + i*array->strides[0] + j*array->strides[1])) = dmat[i + ndim*j];
	}
	else{/* no float or double */
	  return 1;
	}
      }
    }
  }
  return 0;
}

int double2array1d(double *dmat,PyArrayObject *array)
{
  int ndim,i;
  ndim = array->dimensions[0];
  for(i=0;i<ndim;i++){
    if(array->descr->type_num == PyArray_FLOAT){
      (*(float *)(array->data + i*array->strides[0])) = (float)dmat[i];
    }
    else{
      if(array->descr->type_num == PyArray_DOUBLE){
	(*(double *)(array->data + i*array->strides[0])) = dmat[i];
      }
      else{/* no float or double */
	return 1;
      }
    }
  }
  return 0;
}

/* get element contents as float */
float get_element(PyArrayObject *array,int nelm)
{
  if(array->descr->type_num == PyArray_FLOAT){
    return((float)(*(float *)(array->data + nelm*array->strides[0])));
  }
  else{
    if(array->descr->type_num == PyArray_DOUBLE){
      return((float)(*(double *)(array->data + nelm*array->strides[0])));
    }
    else{
      return 0;
    }
  }
}
