/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* dconex.c converted from dconex.f using f2c
$Id$
$Date: 2004-03-08 09:45:33 +0900 (月, 08  3月 2004) $
*/

#include "bld.h"
int dbound_(double*,int*,float*,float*,int*,double*);
int dcon1_(double*,double*, double*,double*,double*,double*,double*, double*,int*);

/*  DCONEX.FTN  COMPUTE CONVOLUTION OF 1 EXPONENTIAL WITH A SAMPLED CURVE */

/*  Last Edit     :  20-sep-88 */
/*  Application   :  BLD */
/*  Author        :  RE CARSON */
/*  Modifications : */
/*  20-sep-88 extrapolated linearly beyond last point */
/*  6-SEP-85  CORRECTED LOW VALUE ROUNDOFF ERRORS IN DCON1 */
/*  22-MAY-85  DCONEX - ZEROED OUTPUT IN CASE OF T1<T0 */
/*  ------------------------------------------------------------------------ */
/*  MODULES IN THIS FILE: DCONEX  DCON1  DBOUND */
/*  ------------------------------------------------------------------------ */

/*  THE MODULES IN THIS SOURCE FILE ARE RESPONSIBLE FOR THE COMPUTATION */
/* 	OF THE VALUE OF A CONVOLUTION INTEGRAL. THIS USED IN BLD FOR */
/* 	THE CALCULATION OF THE EXPECTED TISSUE CURVE FOR THE FDG MODEL, */
/* 	BUT HAS GENERAL APPLICATIONS TO SOLUTIONS OF LINEAR COMPARTMENT */
/* 	MODELS.   THE MAIN ROUTINE IS DCONEX WHICH CALLS SUBROUTINES */
/* 	DBOUND AND DCON1. */

/*   DCONEX IS THE DOUBLE PRECISION VERSION OF RKAREA.  IN FACT, RKAREA */
/* 	PERFORMED ITS CALCULATIONS IN DOUBLE PRECISION, BUT RETURNED */
/* 	THE RESULTS IN SINGLE PRECISION. */

/* DCONEX(TIME,COUNT,IFLG,NP,T0,T1,RK,CV,CA,IERR) - THIS SUBROUTINE COMPUTES */
/* 	ONE OR TWO CONVOLUTION(**) INTEGRALS: */

/*   INTEGRAL FROM T0 TO T1 [ COUNT(s) EXP( -RK*(T1-s) ) ds ] WHICH */
/* 					IS STORED IN CA(1) */
/*   INTEGRAL FROM T0 TO T1 [ s*COUNT(s) EXP( -RK*(T1-s) ) ds ] WHICH */
/* 				IS CALCULATED IF IFLG=2 AND STORED IN CA(2) */
/*  THE INTERPOLATED VALUES OF COUNT(T1) AND T1*COUNT(T1) ARE SUPPLIED */
/* 	IN THE OUTPUT ARRAY CV. */

/*  THE COUNT VARIABLE IS A REAL*4 SAMPLED CURVE SUPPLIED AS A LIST OF TIMES AND */
/* 	COUNTS.  LINEAR INTERPOLATION BETWEEN THE SAMPLED POINTS */
/* 	IS ASSUMED. */

/*   ARGUMENTS: */
/* 	TIME:  LIST OF REAL*4 TIME VALUES FOR COUNT */
/* 	COUNT: LIST OF REAL*4 COUNT VALUES */
/* 	IFLG: IF =2 THE PRODUCT CONVOLUTION IS COMPUTED, OTHERWISE */
/* 		ONLY THE COUNT CONVOLUTION */
/* 	NP:  NUMBER OF POINTS IN TIME/COUNT */
/* 	T0:  STARTING TIME POINT OF CONVOLUTION INTEGRAL */
/* 	T1:  ENDING TIME POINT OF CONVOLUTION INTEGRAL */
/* 	RK:  COEFFICIENT OF EXPONENTIAL TERM */
/* 	CV:  INTERPOLATED VALUES OF COUNT AND TIME*COUNT AT T1 (OUTPUT) */
/* 	CA:  VALUES OF 1 OR 2 CONVOLUTION INTEGRALS (OUTPUT) */
/* 	IERR:  ERROR FLAG. NORMAL =0  -1 = ILLEGAL T0,T1 */
int dconex_(time, count, iflg, np, t0, t1, rk, cv, ca, ierr)
float *time, *count;
int *iflg, *np;
double *t0, *t1, *rk, *cv, *ca;
int *ierr;
{
  /* Local variables */
  static double a;
  static int i;
  static double c0, c1;
  static int n1, n2;
  static double x0, y0, x1, y1, at;

  /* Function Body */
  *ierr = 0;
  if (*t1 < *t0) {
    /*  ERROR EXIT */
    *ierr = -1;
    cv[0] = (float)0.;
    cv[1] = (float)0.;
    ca[0] = (float)0.;
    ca[1] = (float)0.;
    return 0;
  }

/*  FIND THE RANGE OF INDICES IN THE SAMPLED CURVES RELEVANT TO THE */
/* 	BOUNDS OF INTEGRATION (T0,T1).  INTERPOLATE COUNT VALUES AT THE BOUNDS */

    dbound_(t0, np, time, count, &n1, &c0);
    dbound_(t1, np, time, count, &n2, &c1);

/*  ZERO RUNNING SUMS */

    at = (float)0.;
    a = (float)0.;

/*   INTEGRATION LOOP: */
/* 	LOOP POINT TO POINT THROUGH THE (TIME,COUNT) VECTORS.  ASSUME */
/* 	LINEAR INTERPOLATION BETWEEN THE SAMPLED POINTS. */

    x0 = *t0;
/* LEFT TIME OF SEGMENT */
    y0 = c0;
/* LEFT VALUE OF SEGMENT */
    if (n1 != n2) {
      /*  LOOP THROUGH THE LINE SEGMENTS */
      for (i = n1; i < n2; ++i) {
	x1 = time[i];
	/* RIGHT TIME OF SEGMENT */
	y1 = count[i];

	/*  UPDATE INTEGRALS FOR INTERVAL [X0,X1] */
	/* RIGHT VALUE OF SEGMENT */
	if (x0 < x1) {
	  dcon1_(&x0, &y0, &x1, &y1, rk, t1, &a, &at, iflg);
	}

	/*  MOVE TO NEXT LINE SEGMENT */
	x0 = x1;
	y0 = y1;
      }
    }

    /*   INTEGRATE OVER LAST SEGMENT  [X0,T1] */
    y1 = c1;
    if (x0 < *t1) {
	dcon1_(&x0, &y0, t1, &y1, rk, t1, &a, &at, iflg);
    }

/*   STORE FINAL RESULTS */

    cv[0] = y1;
    cv[1] = y1 * *t1;
    ca[0] = a;
    ca[1] = at;
    return 0;

} /* dconex_ */

/* + */

/* DBOUND(T,NP,TIME,COUNT,N,C) - THIS SUBROUTINE WILL LOCATE THE TIME 'T' IN THE */
/* 	SAMPLED CURVE ('TIME','COUNT') OF LENGTH 'NP' AND RETURN THE INDEX OF */
/* 	THE APPROPRIATE LOCATION 'N' AND THE INTERPOLATED VALUE 'C' */

/*    TIME AND COUNT ARE REAL*4, EVERYTHING ELSE IS REAL*8 */
/*  INDEX N IS DEFINED: */
/* 	IF T<TIME(1)			N=0 */
/* 	   TIME(J)<=T<TIME(J+1)		N=J   J=1,...,NP-1 */
/* 	   TIME(NP)<=T			N=NP */
int dbound_(double *t,int *np,float *time,float *count,int *n,double *c)
{
  /* Local variables */
  static double perc, cn, tn, cn1, tn1;



    /* Parameter adjustments */
    --count;
    --time;

    /* Function Body */
    if (*t <= time[1]) {
      /*  VALUE IS BEFORE BEGINNING OF LIST.  ASSUME CURVE IS LINEAR FROM */
      /* 	(0,0) TO (TIME(1),COUNT(1)), AND 0 BEFORE T=0 */

      *n = 0;
      if (*t <= (float)0.) {
	*c = (float)0.;
	return 0;
      }
      *c = *t * (double) count[1] / (double) time[1];
      return 0;
    }


/*  SEARCH THROUGH LIST OF TIMES */
    for (*n = 2; *n <= *np; ++(*n)) {
      if (*t < time[*n]) {
	break;
      }
    }
    if(*n>*np){
      /*   T IS PAST END-OF-LIST. use last 2 points for interpolation */
      *n = *np;
    }

/*  T WAS WITHIN THE LIST */
    --(*n);
/*  right point for interpolation */
    tn1 = time[*n + 1];
    cn1 = count[*n + 1];
/*  left point for interpolation */
    tn = time[*n];
    cn = count[*n];

    perc = (*t - tn) / (tn1 - tn);
/* PROPORTION WITHIN SEGMENT */
    *c = cn + perc * (cn1 - cn);
    return 0;
} /* dbound_ */

/* + */

/*  DCON1(T1,C1,T2,C2,RRK,TT,A,AT,IFLG) - THIS SUBROUTINE COMPUTES ONE OR TWO */
/* 	INTEGRALS OF THE CONVOLUTION OF A STRAIGHT LINE WITH AN EXPONENTIAL. */
/* 	THE STRAIGHT LINE CONNECTS (T1,C1) WITH (T2,C2) (CALL IT g(x)=B+Mx). */
/* 	RRK IS THE EXPONENTIAL RATE CONSTANT. TT IS THE TIME PARAMETER OF */
/* 	THE CONVOLUTION INTEGRAL.  A AND AT (REAL*8) ARE UPDATED WITH THE */
/* 	RESULTS OF THE TWO CONVOLUTIONS.  IFLG=2 INSTRUCTS THE ROUTINE TO */
/* 	COMPUTE THE SECOND INTEGRAL. */

/* 1.  INTEGRAL FROM T1 TO T2 [ g(x) EXP( -RRK*(TT-x) ) dx ] */
/*    =INTEGRAL [ (B+Mx)exp(k(x-T)) ] = exp(k(x-T)) [ B + M(x-1/k) ] /k */

/* 2.   INTEGRAL FROM T1 TO T2 [ x*g(x) EXP( -RRK*(TT-x) ) dx ] */
/* 				IS CALCULATED IF IFLG=2 */
/*     =INTEGRAL [ x(B+Mx)exp(k(x-T) ] = */
/* 					    2	      2 */
/* 		exp(k(x-T)) [ b(x-1/k) + M(x -2x/k+2/k ) ] /k */

/*  THE RESULTS ARE ADDED TO A AND AT. RESPECTIVELY. */
int dcon1_(double *x1,double *y1, double *x2,double *y2,double *rk,double *t,double *a, double *at,int *iflg)
{
  /* System generated locals */
  double d1, d2, d3;

  /* Local variables */
  static double rkdx, b, d, e1, e2, x0, y0, da, dx, dy, rm, dat, dex, rki;



    if (*x1 >= *x2) {
	return 0;
    }

/*  CHECK FOR AND HANDLE AN EXPONENT THAT IS SMALL.  IF SO, USE A TAYLOR */
/* 	EXPANSION FOR THE VALUE */

    dx = *x2 - *x1;
    rkdx = *rk * dx;
    if (fabs(rkdx) <= (float)1e-4) {
      x0 = (*x1 + *x2) / 2.;
      y0 = (*y1 + *y2) / 2.;
      dy = *y2 - *y1;
      dex = exp(-(*rk) * (*t - x0));
      /* Computing 2nd power */
      d1 = rkdx;
      /* Computing 3rd power */
      d2 = rkdx;
      da = dex * dx * (y0 + rkdx * dy / 12 + d1 * d1 * y0 / 24 + d2 * (d2 * d2) * dy / 480);
      *a += da;
      if (*iflg != 2) {
	return 0;
      }
      /* Computing 2nd power */
      d1 = dx;
      /* Computing 2nd power */
      d2 = rkdx;
      /* Computing 3rd power */
      d3 = rkdx;
      dat = dex * (d1 * d1) * (dy / 12 + rkdx * y0 / 12 + d2 * d2 * dy /160 + d3 * (d3 * d3) * y0 / 480);
      *at = *at + dat + x0 * da;
      return 0;
    }

/*  EXPONENT IS NON-ZERO. CALCULATE EXPONENTIAL TERMS */
    rki = 1. / *rk;

/*  CALCULATE THE SLOPE AND INTERCEPT */

    rm = (*y2 - *y1) / (*x2 - *x1);
    b = *y1 - rm * *x1;
    d = *rk * (*x1 - *t);
    if (d > 38.) {
	d = 38.;
    }
    e1 = exp(d);
    d = *rk * (*x2 - *t);
    if (d > 38.) {
	d = 38.;
    }
    e2 = exp(d);

/* 	CALCULATE INTEGRAL 1 AND UPDATE A: */
/*  INTEGRAL [ (B+Mx)exp(k(x-T)) ] = exp(k(x-T)) [ B + M(x-1/k) ] /k */

/*   RE-WRITTEN FOR INCREASED ACCURACY FOR SMALL RK */

/* Computing 2nd power */
    d1 = rki;
    *a = *a + (e2 * (b + rm * *x2) - e1 * (b + rm * *x1)) * rki - rm * (e2 - 
	    e1) * (d1 * d1);

/* 	CALCULATE INTEGRAL 2, IF NECESSARY, AND UPDATE AT: */
/*  INTEGRAL [ x(B+Mx)exp(k(x-T) ] = */
/* 					    2	      2 */
/* 		exp(k(x-T)) [ b(x-1/k) + M(x -2x/k+2/k ) ] /k */
/*  THIS NEXT EQUATION CAN BE RE-WRITTEN FOR INCREASED ACCURACY FOR LOW RK */

    if (*iflg != 2) {
	return 0;
    }
/* Computing 2nd power */
    d1 = *x2;
/* Computing 2nd power */
    d2 = rki;
    d = e2 * (b * (*x2 - rki) + rm * (d1 * d1 - rki * 2 * *x2 + d2 * 
	    d2 * 2));
/* Computing 2nd power */
    d1 = *x1;
/* Computing 2nd power */
    d2 = rki;
    d -= e1 * (b * (*x1 - rki) + rm * (d1 * d1 - rki * 2 * *x1 + d2 * 
	    d2 * 2));
    *at += d * rki;

    return 0;
} /* dcon1_ */

