/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* dminv.c convert from dminv.f using f2c */
/* $Id$ */
/* $Date: 2004-03-05 19:33:15 +0900 (金, 05  3月 2004) $ */
#include "bld.h"

/*  DMINV.FTN - DOUBLE PRECISION MATRIX INVERSION ROUTINE */

/*  Last Edit     :  7-FEB-82 */
/*  Application   :  BLD */
/*  Author        :  IBM SSP / RE CARSON */
/*  Modifications : */
/*  ------------------------------------------------------------------------ */
/*  MODULES IN THIS FILE:  DMINV */
/*  ------------------------------------------------------------------------ */
/*  CALL DMINV(A,N,D,L,M) */

/*      A  REAL MATRIX TO BE INVERTED (DESTROYED, INVERSE RETURNED IN A) */
/* 	   DATA STORE SEQUENTIALLY, BY COLUMNS */
/*      N  ORDER OF SQUARE MATRIX A */
/*      D  DETERMINANT (D=0 MEANS SINGULAR) */
/*      L  INT WORKING VECTOR LENGTH N */
/*      M  INT WORKING VECTOR LENGTH N */

/*   FOR MORE DETAILS ON THE OPERATION OF THIS SUBROUTINE, SEE THE */
/* 	IBM SCIENTIFIC SUBROUTINE DOCUMENTATION. */

int dminv_(double *a,int norder,double *d,double *lvec,double *mvec)
{
  /* System generated locals */
  double d1;

  /* Local variables */
  static double biga, hold;
  static int i, j, k, ij, ki, ji, kk, jk, ik, nk, jp, kj, jq, jr, iz;

  /* Parameter adjustments */
  --a;

  /* Function Body */
  *d = (float)1.;
  nk = -(norder);
  for (k = 1; k <= norder; ++k) {
    nk += norder;
    lvec[k - 1] = k;
    mvec[k - 1] = k;
    kk = nk + k;
    biga = a[kk];
    for (j = k; j <= norder; ++j) {
      iz = norder * (j - 1);
      for (i = k; i <= norder; ++i) {
	ij = iz + i;
	if (fabs(biga) - (d1 = a[ij], fabs(d1)) < 0.) {
	  biga = a[ij];
	  lvec[k - 1] = i;
	  mvec[k - 1] = j;
	}
      }
    }
    j = (int)lvec[k - 1];
    if (j - k > 0) {
      ki = k - norder;
      for (i = 1; i <= norder; ++i) {
	ki += norder;
	hold = -a[ki];
	ji = ki - k + j;
	a[ki] = a[ji];
	a[ji] = hold;
      }
    }
    i = (int)mvec[k - 1];
    if (i - k > 0) {
      jp = norder * (i - 1);
      for (j = 1; j <= norder; ++j) {
	jk = nk + j;
	ji = jp + j;
	hold = -a[jk];
	a[jk] = a[ji];
	a[ji] = hold;
      }
    }
    if (biga == 0.) {
      *d = (float)0.;
      return 0;
    }
    for (i = 1; i <= norder; ++i) {
      if (i - k != 0) {
	ik = nk + i;
	a[ik] /= -biga;
      }
    }
    for (i = 1; i <= norder; ++i) {
      ik = nk + i;
      hold = a[ik];
      ij = i - norder;
      for (j = 1; j <= norder; ++j) {
	ij += norder;
	if (i - k != 0) {
	  if (j - k != 0) {
	    kj = ij - i + k;
	    a[ij] = hold * a[kj] + a[ij];
	  }
	}
      }
    }
    kj = k - norder;
    for (j = 1; j <= norder; ++j) {
      kj += norder;
      if (j - k != 0) {
	a[kj] /= biga;
      }
    }
    if (*d < 1e20) {
      *d *= biga;
    }
    a[kk] = (float)1. / biga;
  }
  k = norder;
  while(1){
    --k;
    if (k <= 0) {
      return 0;
    }
    i = (int)lvec[k - 1];
    if (i - k > 0) {
      jq = norder * (k - 1);
      jr = norder * (i - 1);
      for (j = 1; j <= norder; ++j) {
	jk = jq + j;
	hold = a[jk];
	ji = jr + j;
	a[jk] = -a[ji];
	a[ji] = hold;
      }
    }
    j = (int)mvec[k - 1];
    if (j - k > 0) {
      ki = k - norder;
      for (i = 1; i <=  norder; ++i) {
	ki += norder;
	hold = a[ki];
	ji = ki - k + j;
	a[ki] = -a[ji];
	a[ji] = hold;
      }
    }
 }
} /* dminv_ */

