/* Do integration of sampled curve */
/* Copyright 2000-2020 by H. Watabe */
/* $Id: integrate.c,v 441fdbff584b 2020/10/11 06:03:04 watabe $ */
/*
c  9/3/87  re carson  made this routine work if times X are not
c		in order
c  9/16/86  re carson  initial version
C
C   integrate(N,T,C,n1,X,Y) - THIS SUBROUTINE TAKES THE SET OF REAL*4 POINTS 
C	IN THE ARRAYS T AND C (LENGTH N) AND TREATS THEM AS TIMES AND
C	COUNTS. IT GENERATES THE INTEGRAL OF THAT CURVE AT THE TIME
C	POINTS X (LENGTH N1) INTO THE ARRAY Y.
C	ALL T VALUES less than zero are treated as zero.
C	C(T=0) is assumed to be 0.  BEYOND THE LAST T POINT, THE CURVE IS
C	EXTENDED linearly.
c	NUMERICAL INTEGRATION IS PERFORMED BY TRAPEZOID RULE.
*/
#include <stdio.h>
#include <math.h>
#define MAX(a,b) (a > b ? a : b)

void integrate_(int *n,float *T,float *C,int *n1,float *X,float *Y)
{
  float a,tint,ctint,xx;
  int itc = 0,i;
  float cinterp;

  a = tint = ctint = 0.0;
  for(i=0;i<*n1;i++){
    xx = (float)MAX(X[i],0.0);
    if (xx==0.0) Y[i] = 0.0;
    else{
      /* check if we are still going forward - if not, start at beginning again */
      if(xx<tint){
	a = tint = ctint = 0.0;
	itc = 0;
      }
      /*c  integrate up to first point in curve after point of interest*/
      while((itc<*n)&&(T[itc]<=xx)){
	a += (float)0.5*(C[itc] + ctint)*(T[itc] - tint);
	tint = T[itc];
	ctint = C[itc];
	itc++;
      };
      /*
	c  interpolate count value at time xx
	c	if beyond last point, extrapolate and prevent negatives
      */
      if(itc<*n){
	cinterp = ctint + (C[itc] - ctint)*(xx - tint)/(T[itc] - tint);
      }
      else{
	cinterp = C[*n - 2] + (C[*n - 1] - C[*n - 2])*(xx - T[*n - 2])/(T[*n - 1] - T[*n - 2]);
      }
      Y[i] = (float)(a + 0.5*(cinterp + ctint)*(xx - tint));
    }
  }
}
