/* do interpolation of sampled curve */
/* Copyright 2000-2020 by H. Watabe */
/* $Id: interpolate.c,v 345fef9f1a3d 2020/10/11 05:57:04 watabe $ */
/*
c  9/3/87 re carson  made this routine work if X times are not
c	increasing
c  9/16/86  re carson  initial version
C
C   interpolate(N,T,C,n1,X,Y) - THIS SUBROUTINE TAKES THE SET OF REAL*4 POINTS 
C	IN THE ARRAYS T AND C (LENGTH N) AND TREATS THEM AS TIMES AND
C	COUNTS. IT interpoaltes that curve at
C	POINTS X (LENGTH N1) INTO THE ARRAY z.
C	THIS ROUTINE ASSUMES THAT THE T ARRAY IS SORTED IN
C	ASCENDING ORDER, THAT ALL T VALUES ARE GREATER THAN OR EQUAL TO
C	ZERO, AND THAT C(T=0)=0.  BEYOND THE LAST T POINT, THE CURVE IS
C	EXTENDED linearly.  if x <0, the the result is 0
*/
#include <stdio.h>
#include <math.h>
#include "bld.h"
#define MAX(a,b) (a > b ? a : b)

void interpolate_(int *n, float *T, float *C, int *n1,float *X,float *Z)
{
  float tint,ctint,xx = 0.0;
  int i,itc = 0;
  float cinterp;

  tint = ctint = 0.0;

  for(i=0;i<*n1;i++){
    if(X[i]<0) Z[i] = 0.0;
    else
      xx = (float)MAX(X[i],0.0);
    /* see if x times are increasing*/
    if(xx<tint) {
      tint = 0.0;
      ctint = 0.0;
      itc = 0;
    }
    /*search for first point in curve past point of interest*/
    while((itc<*n)&&(T[itc]<=xx)){
      tint = T[itc];
      ctint = C[itc];
      itc++;
    };

    /* interpolate count value at time xx
       if beyond last point, extrapolate and prevent negatives
    */
    if(itc<*n){
      cinterp = ctint + (C[itc] - ctint)*(xx - tint)/(T[itc] - tint);
    }
    else{
      cinterp = C[*n - 2] + (C[*n - 1] - C[*n - 2])*(xx - T[*n - 2])/(T[*n - 1] - T[*n - 2]);
    }
    Z[i] = cinterp;
  }
}
