/* Copyright (C) 2000-2007 Hiroshi Watabe watabe@ri.ncvc.go.jp */
/* matrixbld.c converted from matrixbld.f using f2c
$Id$
$Date: 2004-03-02 08:43:03 +0900 (火, 02  3月 2004) $
*/
#include "bld.h"
/*  MATRIXBLD.FOR - DOUBLE PRECISION MATRIX MANIPULATION ROUTINES */
/*  Last Edit     :  24-APR-82 */
/*  Application   :  BLD / GENERAL */
/*  Author        :  RE CARSON */
/*  Modifications : */
/*  ------------------------------------------------------------------------ */
/*  MODULES IN THIS FILE: DIAG  DMADD DMCPY DMMUL  DMMULK  DMTRN  DMZER */
/*  ------------------------------------------------------------------------ */

/*  THIS SOURCE FILE CONTAINS AN ASSORTMENT OF MATRIX MANIPULATION SUBROUTINES. */
/* 	AS A GENERAL RULE, THE FIRST ARGUMENT IS THE OUTPUT MATRIX, */
/* 	THE NEXT 1 OR 2 ARGS ARE THE INPUT MATRICES, AND THE REMAINING */
/* 	ARGUMENTS ARE THE DIMENSIONS.  ALL MATRICES ARE ASSUMED TO BE */
/* 	DOUBLE PRECISION FLOATING POINT */

/*  DMADD(A,B,C,M,N) - THIS SUBROUTINE WILL ADD THE 'M' BY 'N' MATRICES 'B' */
/* 	AND 'C' YIELDING 'A' */

int dmadd_(double *a,double *b,double *c, int mno, int nno)
{
  /* Local variables */
  static int i, j;

  /* Function Body */
  for (j = 0; j < nno; ++j) {
    for (i = 0; i < mno; ++i) {
      a[i + j*mno] = b[i + j*mno] + c[i + j*mno];
    }
  }
  return 0;
} /* dmadd_ */

/*  ------------------------------------------------------------------------ */
/* + */

/*  DMMULK(A,B,C,M,N) - THIS SUBROUTINE WILL MULTIPLY THE 'M' BY 'N' MATRIX 'B' */
/* 	BY THE SCALAR 'C' YIELDING 'A' */
int dmmulk_(double *a,double *b, double factor, int mno, int nno)
{
  /* Local variables */
  static int i, j;

  /* Function Body */
  for (j = 0; j < nno; ++j) {
    for (i = 0; i < mno; ++i) {
      a[i + j * mno] = b[i + j * mno]*factor;
    }
  }
  return 0;
} /* dmmulk_ */

/*  ------------------------------------------------------------------------ */
/*  DMCPY(A,B,M,N) - THIS SUBROUTINE WILL COPY THE 'M' BY 'N' MATRIX 'B' */
/* 	INTO THE MATRIX 'A' */
int dmcpy_(double *a,double *b, int mno, int nno)
{
  /* Local variables */
  static int i, j;

  /* Function Body */
  for (j = 0; j < nno; ++j) {
    for (i = 0; i < mno; ++i) {
      a[i + j*mno] = b[i + j*mno];
    }
  }
  return 0;
} /* dmcpy_ */

/*  DMZER(A,M,N) - THIS SUBROUTINE WILL SET THE 'M' BY 'N' MATRIX 'A' */
/* 	TO ZERO */
int dmzer_(double *a, int mno,int nno)
{
  /* Local variables */
  static int i, j;
  /* Function Body */
  for (j = 0; j < nno; ++j) {
    for (i = 0; i < mno; ++i) {
	a[i + j * mno] = 0.0;
    }
  }
  return 0;
} /* dmzer_ */

/*  DMTRN(A,B,M,N) - THIS SUBROUTINE WILL WRITE THE TRANSPOSE OF THE 'M' BY 'N' */
/* 	MATRIX 'B' INTO THE MATRIX 'A' */
int dmtrn_(double *a, double *b,int mno, int nno)
{
  /* Local variables */
  static int i, j;

  /* Function Body */
  for (j = 0; j < mno; ++j) {
    for (i = 0; i < nno; ++i) {
      a[i + j * mno] = b[j + i * nno];
    }
  }
  return 0;
} /* dmtrn_ */

/*  DMMUL(A,B,C,M,N) - THIS SUBROUTINE WILL MULTIPLY THE 'L' BY 'M' MATRIX 'B' */
/* 	TIMES THE 'M' BY 'N' MATRIX 'C'	YIELDING THE 'L' BY 'N' MATRIX 'A' */
int dmmul_(double *a, double *b,double *c, int lno,int mno,int nno)
{
  /* Local variables */
  static int i, j, k;
  static double t;

  /* Function Body */
  for (i = 0; i < lno; ++i) {
    for (j = 0; j < nno; ++j) {
      t = 0.0;
      for (k = 0; k < mno; ++k) {
	t += b[i + k * lno] * c[k + j * mno];
      }
      a[i + j * lno] = t;
    }
  }
  return 0;
} /* dmmul_ */

/*  DIAG(D,A,M) - THIS SUBROUTINE WILL EXTRACT THE DIAGONAL OF THE 'M' BY 'M' */
/* 	MATRIX 'A' AND LOAD IT INTO THE VECTOR 'D' */
int diag_(double *d, double *a, int mno)
{
  /* Local variables */
  static int i;

  /* Function Body */
  for (i = 0; i < mno; ++i) {
    d[i] = a[i + i * mno];
  }
  return 0;
} /* diag_ */

