/* pconv_exp.c product convolution exp function */
/* Copyright 2000-2007 by H. Watabe */
/* $Id$ */
/*
C  CONV_exp.For  conv_exp function for var
C  LAST EDIT 10/1/87
c
c  10/1/87  re carson allowed x values to be out of order
c  9/16/86 re carson  created from convlv external task
C
C
C  THIS ROUTINE WILL COMPUTE:
C	[P(1)*exp(-P(2)*X) + P(3)*exp(-P(4)*X) + ... ] *** B(X)
C		WHERE *** IS CONVOLUTION.
C  ARGUMENTS:
C	(A,B)  INPUT FUNCTION
C	P	LIST OF PARAMETERS
C		IF LIST IS ODD LENGTH, ASSUME LAST P IS A CONSTANT
C	X	TIMES AT WHICH TO COMPUTE CONVOLUTIONS
C
C	ASSUME A AND X ARE SORTED ASCENDINGLY
C
*/
#include <stdio.h>
#include <math.h>
#include "bld.h"

void pconvexp(int *nx,float *x, int *npar, float *p, int *nbld, float *A, float *B, float *ans)
{
  float cst;
  double cv[2],ca[2],t0,t1,rk,xint,dt;
  unsigned char even;
  int nexp,i,j,ierr,itmp;

  nexp = (int)((*npar + 1)/2.0);
  even = (*npar)%2 ? 0:1;
  /*C  ZERO RESPONSE VECTOR*/
  for(j=0;j<*nx;j++){
    ans[j] = 0.0;
  }
  /*
C  EVALUATE FUNCTION FOR THE NEXP EXPONENTS
C
C  EVALUATE CONVOLUTION ITERATIVELY, NOTING THAT IF
C	Q(T0,T1)=INTEGRAL FROM T0 TO T1 (exp(-RK*(T1-t) * g(t))
C  THEN Q(0,B)=Q(A,B) + exp(-RK*(B-A)) * Q(0,A)
  */
  for(j=1;j<=nexp;j++){
    cst = p[2*j - 2];
    if((j==nexp)&&(even==0)){
      rk = 0.0;
    }
    else{
      rk = p[2*j - 1];
    }
    t0 = 0.0;
    xint = 0.0;
    for(i=1;i<=*nx;i++){
      t1 = x[i - 1];
      dt = t1 - t0;
      if(dt<0){
	t0 = 0.0;
	itmp = 2;
	dconex_(A,B,&itmp,nbld,&t0,&t1,&rk,cv,ca,&ierr);
	xint = ca[1];
      }
      else{
	if(dt>=0.0){
	  itmp = 2;
	  dconex_(A,B,&itmp,nbld,&t0,&t1,&rk,cv,ca,&ierr);
	  xint = exp(-rk*dt)*xint + ca[1];
	}
      }
      ans[i - 1] = (float)(ans[i - 1] + cst*xint);
      t0 = t1;
    }
  }
}
